var Canvas = require("canvas");
var BlockFont = require("../scriptel-blockfont.js");

/**
 * This plugin will intercept KeyboardInputRequest inbound messages
 * and will cause the leased device to enter "keyboard mode" which will
 * stop all pen and button events while input is entered. Upon completion
 * the device will be reset.
 */
function KeyboardPlugin() {
    this.version = "1.0.0";
    this.name = "Keyboard Plugin";
    this.description = "This plugin enables an on-screen keyboard on ScripTouch events for textual input.";
    this.author = "Michael Powers <mpowers@scriptel.com>";

    this.defaultKeyPages = [
        [
            "qwertyuiop",
            "asdfghjkl",
            "zxcvbnm"
        ],
        [
            "01234@",
            "56789."
        ],
        [
            "+x=<>{}[]%",
            "!#$/^&*()~`",
            "-'\":;,?\\|"
        ]
    ];

    this.keyboards = {};
}
function KeyboardInstance() {
    this.keyPages = this.defaultKeyPages;
    this.keys = [];
    this.canvas = undefined;
    this.ctx = undefined;
    this.shift = false;
    this.query = "";
    this.response = "";
    this.okCaption = "";
    this.width = 240;
    this.height = 64;
    this.page = 0;
    this.protocolDriver = undefined;
    this.lease = undefined;
    this.state = undefined;
    this.selectedKey = undefined;
    this.inKeyboard = false;
    this.injectPosition = 0;

    this.resetToDefault = true;
    this.maskInput = false;
    this.formatString = "";
    this.sizeLimit = false;
    this.disableSpace = false;
    this.disableShift = false;
}
KeyboardPlugin.prototype.getInstance = function(path) {
    var instance;
    for(instance in this.keyboards) {
        if(this.keyboards.hasOwnProperty(instance)) {
            if(instance===path) {
                return this.keyboards[instance];
            }
        }
    }
    return false;
}
KeyboardPlugin.prototype.handleInboundMessage = function(message, connection, state, lease) {
    if(message._class==="KeyboardRequest") {
        if(!state || !lease) {
            connection.sendUTF(JSON.stringify({"_class":"KeyboardException","message":"No device currently leased."}));
            return;
        }
        var instance = new KeyboardInstance();
        instance.protocolDriver = lease.deviceInfo.protocolDriver;
        instance.state = state;
        instance.lease = lease;
        instance.width = state.deviceInfo.displayInfo.width;
        instance.height = state.deviceInfo.displayInfo.height;
        instance.canvas = new Canvas(instance.width,instance.height);
        instance.ctx = instance.canvas.getContext("2d");

        instance.query = (message.query)?message.query:"Please Respond:";
        instance.keyPages = (message.keyPages)?message.keyPages:this.defaultKeyPages;
        instance.okCaption = (message.okCaption)?message.okCaption:"Ok";
        
        instance.resetToDefault = (message.resetToDefault)?message.resetToDefault:false;
        instance.maskInput = (message.maskInput)?message.maskInput:false;
        instance.formatString = (message.formatString)?message.formatString:"";
        instance.sizeLimit = (message.sizeLimit)?message.sizeLimit:false;
        instance.disableSpace = (message.disableSpace)?message.disableSpace:false;
        instance.disableShift = (message.disableShift)?message.disableShift:false;

        instance.response = "";
        instance.injectPosition = 0;
        instance.page = 0;

        instance.inKeyboard = true;
        this.keyboards[lease.deviceInfo.path] = instance;
        this.prepareRegions(instance);
        this.drawKeyboard(instance);
        this.drawResponse(instance);
    }
};
KeyboardPlugin.prototype.handleOutboundMessage = function(message, connection, state, lease) {
    var instance = (lease)?this.getInstance(lease.deviceInfo.path):false;
    if(instance && instance.inKeyboard && (message._class==="PenDown" || message._class==="PenMove" || message._class==="PenUp")) {
        var key = this.locateKey(message, instance);
        if(key && message._class==="PenUp") {
            if(instance.selectedKey) {
                this.drawKey(key, false, instance);
                instance.selectedKey = false;
            }

            //Lets see if this is a special key.
            if(key.action && key.action==="backspace") {
                this.removeCharacter(instance);
            } else if(key.action && key.action==="space") {
                this.injectCharacter(" ", instance);
            } else if(key.action && key.action==="pageflip") {
                instance.page = (instance.page + 1) % instance.keyPages.length;
                this.drawKeyboard(instance);
            } else if(key.action && key.action==="shift") {
                instance.shift = !instance.shift;
                this.drawKeyboard(instance);
            } else if(key.action && key.action==="ok") {
                if(instance.resetToDefault) {
                    instance.protocolDriver.resetScreen(instance.lease.deviceHandle, instance.state.deviceInfo);
                } else {
                    instance.protocolDriver.clearScreen(instance.lease.deviceHandle, instance.state.deviceInfo);
                }
                instance.inKeyboard = false;
                connection.sendUTF(JSON.stringify({"_class":"KeyboardResponse","response":instance.response}));
                delete this.keyboards[lease.deviceInfo.path];
                return true;
            } else {
                this.injectCharacter(key.char, instance);
            }
            this.drawResponse(instance);
        } else if(key) {
            if(!instance.selectedKey) {
                this.drawKey(key, true, instance);
                instance.selectedKey = key;
            } else if(instance.selectedKey.char !== key.char) {
                this.drawKey(instance.selectedKey, false, instance);
                this.drawKey(key, true, instance);
                instance.selectedKey = key;
            }
        } else if(!key && instance.selectedKey) {
            this.drawKey(instance.selectedKey, false, instance);
            instance.selectedKey = false;
        }

        return true;
    }

    if(instance && message._class==="DeviceCloseResponse") {
        //Device got closed, but we have a keyboard. Clean it up!
        delete this.keyboards[lease.deviceInfo.path];
    }
};
KeyboardPlugin.prototype.injectCharacter = function(chr, instance) {
    if(instance.sizeLimit && (instance.response.length + 1 > instance.sizeLimit)) {
        //Can't go over our size limit.
        return;
    }

    var tmp = instance.response.substring(0,instance.injectPosition);
    tmp += chr;
    tmp += instance.response.substring(instance.injectPosition);
    instance.response = tmp;
    instance.injectPosition += chr.length;
};
KeyboardPlugin.prototype.removeCharacter = function(instance) {
    if(instance.injectPosition === 0) {
        return;
    }
    var tmp = instance.response.substring(0,instance.injectPosition-1);
    tmp += instance.response.substring(instance.injectPosition);
    instance.response = tmp;
    instance.injectPosition--;
}
KeyboardPlugin.prototype.drawKey = function(key, invert, instance) {
    var canvas = new Canvas(key.width,key.height);
    var ctx = canvas.getContext("2d");
    ctx.fillStyle = "white";
    ctx.fillRect(0,0,key.width,key.height);

    if(key.action) {
        ctx.fillStyle = (invert)?"black":"white";
        ctx.fillRect(-0.5,-0.5,key.width,key.height);

        ctx.strokeStyle = "black";
        if(key.x===0) {
            ctx.strokeRect(0,-0.5,key.width+0.5,key.height);
        } else {
            ctx.strokeRect(-0.5,-0.5,key.width,key.height);
        }
    } else {
        ctx.fillStyle = (invert)?"black":"white";
        ctx.fillRect(1.5,1.5,key.width-1.5,key.height-1.5);

        ctx.strokeStyle = "black";
        ctx.strokeRect(1.5,1.5,key.width-1.5,key.height-1.5);
    }

    var color = (invert)?[0xFF,0xFF,0xFF]:[0,0,0];
    BlockFont.fillText(ctx,key.char,key.tX-key.x,key.tY-key.y,"center",color);

    var sprites = instance.protocolDriver.convertBitmapToSpriteBW(canvas);
    for(i=0;i<sprites.length;i++) {
        var sprite = sprites[i];
        sprite.x += key.x;
        sprite.y += key.y;
        instance.protocolDriver.sendSprite(instance.lease.deviceHandle, instance.state.deviceInfo, sprite);
    }
};
KeyboardPlugin.prototype.drawResponse = function(instance) {
    var canvas = new Canvas(instance.width-2,9);
    var ctx = canvas.getContext("2d");
    ctx.fillStyle = "white";
    ctx.fillRect(0,0,instance.width-2,9);

    var text = instance.response;

    if(instance.maskInput) {
        //Replace all but the last character with astrisks.
        var tmp = "";
        for(var i=0;i<text.length-1;i++) {
            tmp += "*";
        }
        tmp += text.charAt(text.length-1);
        text = tmp;
    }

    if(instance.formatString!="") {
        text = this.applyFormatString(instance.formatString,text);
    }

    var tWidth = BlockFont.calculateWidth(text);
    var shrunk = false;

    while(tWidth + 10 > instance.width) {
        text = text.substring(1);
        tWidth = BlockFont.calculateWidth("..."+text);
        shrunk = true;
    }

    if(shrunk) {
        text = "..."+text;
    }

    BlockFont.fillText(ctx,text,1,2,"left",[0,0,0]);

    //Draw cursor:
    ctx.fillStyle = "black";
    ctx.fillRect(tWidth+2,0,5,9);

    var sprites = instance.protocolDriver.convertBitmapToSpriteBW(canvas);
    for(i=0;i<sprites.length;i++) {
        var sprite = sprites[i];
        sprite.x += 1;
        sprite.y += 10;
        instance.protocolDriver.sendSprite(instance.lease.deviceHandle, instance.state.deviceInfo, sprite);
    }
};
KeyboardPlugin.prototype.locateKey = function(evt, instance) {
    var i;
    for(i=0;i<instance.keys.length;i++) {
        var key = instance.keys[i];
        if(evt.x >= key.x && evt.x <= key.x + key.width && evt.y >= key.y && evt.y <= key.y + key.height) {
            return key;
        }
    }
    return false;
};
KeyboardPlugin.prototype.prepareRegions = function(instance) {
    var region = {
        "id": 2,
        "x1": 0,
        "x2": instance.state.deviceInfo.displayInfo.width-1,
        "y1": 0,
        "y2": instance.state.deviceInfo.displayInfo.height-1,
        "containerType": {
            "_class":"RegionContainer",
            "flags": {
                "inkEnabled": false
            }
        },
        "flags": {
            "acceptsTouch": true,
            "decodeRegion": true,
            "hasFrame": false,
            "inUse": true,
            "visible": true
        }
    };
    instance.protocolDriver.updateRegion(instance.lease.deviceHandle, instance.state.deviceInfo, region);

    for(x=2;x<instance.state.deviceInfo.regions.length;x++) {
        region = {
            "id": (x+1),
            "x1": 0,
            "x2": 1,
            "y1": 0,
            "y2": 1,
            "containerType": {"_class":"RegionContainer"},
            "flags": {
                "acceptsTouch": false,
                "decodeRegion": false,
                "hasFrame": false,
                "inUse": false,
                "visible": false
            }
        };
        instance.protocolDriver.updateRegion(instance.lease.deviceHandle, instance.state.deviceInfo, region);
    };
};
KeyboardPlugin.prototype.drawKeyboard = function(instance) {
    var ctx = instance.ctx;
    ctx.fillStyle = "white";
    ctx.strokeStyle = "black";

    ctx.fillRect(0,0,instance.width,instance.height);
    instance.keys = [];

    //Draw Question
    BlockFont.fillText(ctx,instance.query,0,1,"left",[0,0,0],instance.width);

    //Text Entry Box
    ctx.strokeRect(0,9.5,instance.width,10);

    var keyIdx = 0;
    //Shift
    if(!instance.disableShift) {
        ctx.strokeRect(0.5,54.5,30,9);
        var color = [0,0,0];
        if(instance.shift) {
            ctx.fillStyle = "black";
            ctx.fillRect(0.5,54.5,30,9);
            color = [0xFF,0xFF,0xFF];
        }
        BlockFont.fillText(ctx,"\u2191",15,56,"center",color);
        instance.keys[keyIdx++] = {
            "char": "\u2191",
            "x": 0,
            "y": 55,
            "tX": 15,
            "tY": 56,
            "width": 30,
            "height": 9,
            "action":"shift"
        };
    }

    if(!instance.disableSpace) {
        //Space Bar
        ctx.strokeRect(70.5,54.5,100,9);
        BlockFont.fillText(ctx,"Space",120,56,"center");
        instance.keys[keyIdx++] = {
            "char": "Space",
            "x": 71,
            "y": 55,
            "tX": 120,
            "tY": 56,
            "width": 100,
            "height": 9,
            "action":"space"
        };
    }

    //Ok
    ctx.strokeRect(209.5,54.5,30,9);
    BlockFont.fillText(ctx,instance.okCaption,223,56,"center");
    instance.keys[keyIdx++] = {
        "char": instance.okCaption,
        "x": 210,
        "y": 55,
        "tX": 223,
        "tY": 56,
        "width": 30,
        "height": 9,
        "action":"ok"
    };

    //Backspace
    ctx.strokeRect(175.5,54.5,30,9);
    BlockFont.fillText(ctx,"\u2190",190,56,"center");
    instance.keys[keyIdx++] = {
        "char": "\u2190",
        "x": 176,
        "y": 55,
        "tX": 190,
        "tY": 56,
        "width": 30,
        "height": 9,
        "action":"backspace"
    };

    //Key Page
    if(instance.keyPages.length>1) {
        ctx.strokeRect(35.5,54.5,30,9);
        BlockFont.fillText(ctx,(instance.page+1)+"/"+instance.keyPages.length,51,56,"center");
        instance.keys[keyIdx++] = {
            "char": (instance.page+1)+"/"+instance.keyPages.length,
            "x": 36,
            "y": 55,
            "tX": 51,
            "tY": 56,
            "width": 30,
            "height": 9,
            "action":"pageflip"
        };
        if(instance.selectedKey && instance.selectedKey.action==="pageflip") {
            instance.selectedKey = instance.keys[3];
        }
    }

    //Now to draw the current key page.
    var keyPage = instance.keyPages[instance.page];
    var keyHeight = Math.floor((instance.height - 31) / keyPage.length);
    var r,i;
    for(r=0;r<keyPage.length;r++) {
        var page = keyPage[r];
        var keyWidth = Math.floor(instance.width / page.length);
        var offsetLeft = Math.floor((instance.width - (keyWidth * page.length))/2);
        for(i=0;i<page.length;i++) {
            var x = (i*keyWidth) + offsetLeft;
            var y = 20 + (r*keyHeight);
            ctx.strokeRect(x+1.5,y+1.5,keyWidth-2,keyHeight-2);

            var tX = x+Math.floor(keyWidth/2);
            var tY = (y+Math.floor(keyHeight/2))-2;

            var chr = page.charAt(i);
            if(instance.shift) {
                chr = (chr===chr.toUpperCase())?chr.toLowerCase():chr.toUpperCase();
            }

            BlockFont.fillText(ctx,chr,tX,tY,"center");

            var keyInfo = {
                "char": chr,
                "x": x,
                "y": y,
                "tX": tX,
                "tY": tY,
                "width": keyWidth,
                "height": keyHeight
            };

            instance.keys[instance.keys.length] = keyInfo;
        }
    }

    var sprites = instance.protocolDriver.convertBitmapToSpriteBW(instance.canvas);
    for(i=0;i<sprites.length;i++) {
        instance.protocolDriver.sendSprite(instance.lease.deviceHandle, instance.state.deviceInfo, sprites[i]);
    }
};
KeyboardPlugin.prototype.applyFormatString = function(formatString, str) {
    //We need to inject any non-space characters at the correct
    //positions in the string.
    var len = (formatString.length<str.length)?formatString.length:str.length;
    var i;
    for(i=0;i<len;i++) {
        var fc = formatString.charAt(i);
        if(fc!=" ") {
            var tmp = str.substring(0,i);
            tmp += fc;
            tmp += str.substring(i);
            str = tmp;
            if(len+1 < formatString.length) {
                len++;
            }
        }
    }
    return str;
};


var keyboard = new KeyboardPlugin();
module.exports = keyboard;