var dgram  = require('dgram');
var dto    = require("../lib/scriptel-dto.js");
var os     = require("os");
var Logger = require("../lib/scriptel-logger.js");

function ScriptelDiscovery(config) {
    "use strict";
    Logger.setConfiguration(config);
    this.config = config;
    this.callback = false;
    this.sockets = [];
    this.first = false;

    var t = this;

    this.rebindInterfaces(config);

    if(config.discovery.rebindTime > 0) {
        setInterval(function() {
            if(!t.first) {
                Logger.info("Attempting to rebind UDP Multicast Sockets.");
                t.rebindInterfaces(config);
            } else {
                t.first = false;
            }
        }, config.discovery.rebindTime);
    }

	//Catch "nice" termination signals and shut down as gracefully as possible.
	process.on('SIGINT', function () {
		console.log("Caught SIGINT, attempting to shut down gracefully.");
		var shutdown = new dto.InstanceShutdown();
		shutdown.hostName = os.hostname();
		t.sendMessage(shutdown, function () {
            t.unbindInterfaces(config);
			process.exit();
		});
	});
	process.on('SIGTERM', function () {
		console.log("Caught SIGTERM, attempting to shut down gracefully.");
		var shutdown = new dto.InstanceShutdown();
		shutdown.hostName = os.hostname();
		t.sendMessage(shutdown, function () {
            t.unbindInterfaces(config);
			process.exit();
		});
	});
}
ScriptelDiscovery.prototype.unbindInterfaces = function(config) {
    var i;
    for(i = this.sockets.length - 1; i >= 0; i--) {
        var socket = this.sockets[i].socket;
        var socketAddr = this.sockets[i].iface.address;
        socket.close();
        Logger.info("Closed socket with interface {0} from UDP Multicast on {1}:{2}", [socketAddr, config.discovery.address, config.discovery.port]);
        this.sockets.splice(i, 1);
    }
}
ScriptelDiscovery.prototype.rebindInterfaces = function(config) {
    this.unbindInterfaces(config);
    var listenAddr = config.discovery.listenAddress;
    var ifaces = os.networkInterfaces();

    var ifName, i;
    var t = this;
    for(ifName in ifaces) {
        for(i = 0; i < ifaces[ifName].length; i++) {
            var addr = ifaces[ifName][i];
            var bind = {"socket": false, "iface": addr};

            if(addr.family == "IPv4" && (listenAddr === "*" || listenAddr === addr.address)) {
                bind.socket = dgram.createSocket((addr.family=="IPv4")?"udp4":"udp6", function(msg, info) {
                    t.receiveMessage(msg, info);
                });

                var udpBindCallback = function(sock, sockAddr, sockFamily) {
                    Logger.info("Binding socket with interface {0} for UDP Multicast on {1}:{2}: TTL: {3}, MulticastTTL: {4}, family: {5}", [sockAddr, config.discovery.address, config.discovery.port, config.discovery.ttl, config.discovery.multicastTTL, sockFamily]);
                    bind.socket.bind(config.discovery.port, addr.address, function() {
                        sock.setTTL(config.discovery.ttl);
                        sock.setMulticastTTL(config.discovery.multicastTTL);
                        sock.unref();
                        sock.addMembership(config.discovery.address, sockAddr);
                    });
                }

                udpBindCallback(bind.socket, addr.address, addr.family);

                t.sockets[t.sockets.length] = bind;
            }
        }
    }
}
ScriptelDiscovery.prototype.registerCallback = function (f) {
    "use strict";
    this.callback = f;
};
ScriptelDiscovery.prototype.receiveMessage = function (msg, info) {
    "use strict";
    msg = this.bufferToObject(msg);
    if (msg) {
        this.callback(msg, info);
    }
};
ScriptelDiscovery.prototype.broadcastInstanceInformation = function (devices) {
    "use strict";
    var ins = new dto.Instance();
    var x;
    ins.socketPort = this.config.webServer.httpPort;
    ins.sslSocketPort = this.config.webServer.httpsPort;
    ins.hostName = os.hostname();
    ins.devices = [];
    for (x = 0; x < devices.length; x++) {
        ins.devices[ins.devices.length] = devices[x].uuid;
    }
    this.sendMessage(ins);
};
ScriptelDiscovery.prototype.sendMessage = function (msg, callback) {
	"use strict";
    var str = JSON.stringify(msg);

	//We're also going to short-circuit this (just in case broadcasting is disabled
	//on the network we're on, users should always be able to find themselves).
	this.receiveMessage(str, {"address": "localhost", "port": this.config.discovery.port});

    var buffer = new Buffer(str);
    var i;
    for(i = 0; i < this.sockets.length; i++) {
        this.sockets[i].socket.send(buffer, 0, buffer.length, this.config.discovery.port, this.config.discovery.address, callback);
    }
};
ScriptelDiscovery.prototype.bufferToObject = function (buf) {
    "use strict";
    try {
        var obj = JSON.parse(buf.toString());
        return obj;
    } catch (e) {
        Logger.warning("Invalid encoded message received via broadcast: {0}", [e]);
    }
    return false;
};
module.exports = ScriptelDiscovery;

