/**
 * This module contains all of the data transfer objects used to communicate with
 * external processes over the network.
 * @author Scriptel Engineering <EngineeringDepartment@scriptel.com>
 * @module dto
 */

 /**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing information about the
 * digitizer's current kernel version.
 * @constructor
 */
function KernelVersion() {
    "use strict";
    /**
     * Stores the major version of this kernel.
     * @public
     * @instance
     * @type {number}
     */
    this.major = -1;
    /**
     * Stores the minor version of this kernel.
     * @public
     * @instance
     * @type {number}
     */
    this.minor = -1;
    /**
     * Stores the release version of this kernel.
     * @public
     * @instance
     * @type {number}
     */
    this.release = -1;
    this._class = "KernelVersion";
}
module.exports.KernelVersion = KernelVersion;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing information about the
 * digitizer's current firmware version.
 * @constructor
 */
function FirmwareVersion() {
    "use strict";
    /**
     * Stores the major version of this firmware.
     * @public
     * @instance
     * @type {number}
     */
    this.major = -1;
    /**
     * Stores the minor version of this firmware.
     * @public
     * @instance
     * @type {number}
     */
    this.minor = -1;
    /**
     * Stores the release version of this firmware.
     * @public
     * @instance
     * @type {number}
     */
    this.release = -1;
    this._class = "FirmwareVersion";
}
module.exports.FirmwareVersion = FirmwareVersion;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing an object representing a
 * physical digitizer.
 * @tutorial 04 - Handling Pen Events
 * @tutorial 05 - Modifying the Screen
 * @tutorial 06 - Forcing Calibration
 * @constructor
 */
function ScriptelDevice() {
    "use strict";
    /**
     * The UUID of the device.
     * @public
     * @instance
     * @type {string}
     */
    this.uuid = "";
    /**
     * The serial number of the device (if set).
     * @public
     * @instance
     * @type {string}
     */
    this.serialNumber = "";
    /**
     * Information about the physical display installed on this device (if one
     * is installed).
     * @public
     * @instance
     * @type {module:dto~DisplayInfo}
     */
    this.displayInfo = undefined;
    /**
     * Information about the software installed on this device.
     * @public
     * @instance
     * @type {module:dto~SoftwareVersion}
     */
    this.softwareVersion = undefined;
    /**
     * Information about the current configuration for the installed display on
     * the device (if one is installed).
     * @public
     * @instance
     * @type {module:dto~DisplaySettings}
     */
    this.displaySettings = undefined;
    /**
     * Contains information about whether or not this device is currently in
     * calibration mode or not and if so what part of calibration.
     * @public
     * @instance
     * @type {module:dto~CalibrationMode}
     */
    this.calibrationMode = undefined;
    /**
     * Contains information about the regions store in device memory.
     * @public
     * @instance
     * @type {Array<module:dto~Region>}
     */
    this.regions = [];

    /**
     * Contains information about the error correction information built into
     * the device.
     * @public
     * @instance
     * @type {module:dto~ScreenErrorCorrection}
     */
    this.errorCorrection = undefined;

    this._class = "ScriptelDevice";
}
module.exports.ScriptelDevice = ScriptelDevice;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing an object representing the
 * current state of a calibration session.
 * @tutorial 06 - Forcing Calibration
 * @constructor
 */
function CalibrationMode() {
    "use strict";
    /**
     * Flag to determine whether or not the device is currently in calibration mode.
     * @public
     * @instance
     * @type {boolean}
     */
    this.calibrating = false;
    /**
     * Contains a description of the point in calibration mode the device is
     * currently waiting for. Only valid if calibrating is true.
     * @public
     * @instance
     * @type {string}
     */
    this.calibrationPoint = "";
}
module.exports.CalibrationMode = CalibrationMode;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing a region. A firmware 
 * construct used by ST Series digitizers to affect what is displayed on the
 * screen of the device as well how the device behaves.
 * @tutorial 05 - Modifying the Screen
 * @constructor
 */
function Region() {
    "use strict";
    /**
     * Region number as stored in the device.
     * @public
     * @instance
     * @type {number}
     */
    this.id = -1;
    /**
     * Button specific information about this region. If this region is not a
     * button this will be undefined.
     * @public
     * @instance
     * @type {module:dto~RegionButton}
     */
    this.buttonType = undefined;
    /**
     * Container specific information about this region. If this region is not a
     * container this will be undefined.
     * @public
     * @instance
     * @type {module:dto~RegionContainer}
     */
    this.containerType = undefined;
    /**
     * Line specific information about this region. If this region is not a line
     this will be undefined.
     * @public
     * @instance
     * @type {module:dto~RegionLine}
     */
    this.lineType = undefined;
    /**
     * An object containing flags associated with this region.
     * @public
     * @instance
     * @type {module:dto~RegionFlags}
     */
    this.flags = undefined;
    /**
     * The left-most coordinate of this region.
     * @public
     * @instance
     * @type {number}
     */
    this.x1 = -1;
    /**
     * The top-most coordinate of this region.
     * @public
     * @instance
     * @type {number}
     */
    this.y1 = -1;
    /**
     * The right-most coordinate of this region.
     * @public
     * @instance
     * @type {number}
     */
    this.x2 = -1;
    /**
     * The bottom-most coordinate of this region.
     * @public
     * @instance
     * @type {number}
     */
    this.y2 = -1;
    /**
     * The foreground color of this region stored as a four length number array.
     * Stored as: red (0-255), green (0-255), blue (0-255) and alpha (0-255).
     * @public
     * @instance
     * @type {Array<number>}
     */
    this.foregroundColor = [];
    /**
     * The background color of this region stored as a four length number array.
     * Stored as: red (0-255), green (0-255), blue (0-255) and alpha (0-255).
     * @public
     * @instance
     * @type {Array<number>}
     */
    this.backgroundColor = [];
    this._class = "Region";
}
module.exports.Region = Region;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing flags associated with all
 * types of regions
 * @tutorial 05 - Modifying the Screen
 * @constructor
 */
function RegionFlags() {
    "use strict";
    /**
     * Will putting the pen down in this region trigger touch events?
     * @public
     * @instance
     * @type {boolean}
     */
    this.acceptsTouch = false;
    /**
     * Should the device decode this region?
     * @public
     * @instance
     * @type {boolean}
     */
    this.decodeRegion = false;
    /**
     * Will this region be painted by the device?
     * @public
     * @instance
     * @type {boolean}
     */
    this.visible = false;
    /**
     * Is this region currently in use by the device?
     * @public
     * @instance
     * @type {boolean}
     */
    this.inUse = false;
    /**
     * Will the device draw a border around this region?
     * @public
     * @instance
     * @type {boolean}
     */
    this.hasFrame = false;
    this._class = "RegionFlags";
}
module.exports.RegionFlags = RegionFlags;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing container-specific region
 * data associated with a parent region.
 * @tutorial 05 - Modifying the Screen
 * @constructor
 */
function RegionContainer() {
    "use strict";
    /**
     * Contains the flags specific to this region.
     * @public
     * @instance
     * @type {module:dto~RegionContainerFlags}
     */
    this.flags = undefined;
    this._class = "RegionContainer";
}
module.exports.RegionContainer = RegionContainer;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing flags specific to regions
 * that are tagged as containers.
 * data associated with a parent region.
 * @tutorial 05 - Modifying the Screen
 * @constructor
 */
function RegionContainerFlags() {
    "use strict";
    /**
     * Will putting the pen down in this region cause ink to be drawn in the
     * area?
     * @public
     * @instance
     * @type {boolean}
     */
    this.inkEnabled = false;
}
module.exports.RegionContainerFlags = RegionContainerFlags;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing button-specific region data
 * associated with a parent region.
 * @tutorial 05 - Modifying the Screen
 * @constructor
 */
function RegionButton() {
    "use strict";
    /**
     * The number of the parent container that this button should be drawn in.
     * This MUST be set correctly. Not setting this correctly will either cause
     * the button not to appear correctly (with newer firmware) or will cause
     * the device to crash.
     * @public
     * @instance
     * @type {number}
     */
    this.parentContainer = -1;
    /**
     * Stores flags associated with this button.
     * @public
     * @instance
     * @type {module:dto~RegionButtonFlags}
     */
    this.flags = undefined;
    /**
     * Stores the caption for this button. Captions will be limited to the
     * maximum size specified in DisplayInfo.captionLength.
     * @public
     * @instance
     * @type {string}
     */
    this.caption = "";
    this._class = "RegionButton";
}
module.exports.RegionButton = RegionButton;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing flags associated with
 * button regions.
 * @tutorial 05 - Modifying the Screen
 * @constructor
 */
function RegionButtonFlags() {
    "use strict";
    /**
     * Is this button enabled?
     * @public
     * @instance
     * @type {boolean}
     */
    this.enabled = false;
    /**
     * Will this button highlight (invert colors) on select?
     * @public
     * @instance
     * @type {boolean}
     */
    this.highlightOnSelect = false;
    /**
     * Will pressing this button cause the screen to clear?
     * @public
     * @instance
     * @type {boolean}
     */
    this.clearOnSelect = false;
    /**
     * Will pressing this button cause a confirm event?
     * @public
     * @instance
     * @type {boolean}
     */
    this.confirmOnSelect = false;
}
module.exports.RegionButtonFlags = RegionButtonFlags;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing line-specific region data
 * associated with
 * a parent region.
 * @tutorial 05 - Modifying the Screen
 * @constructor
 */
function RegionLine() {
    "use strict";
    /**
     * Parent container id associated with this line.
     * @public
     * @instance
     * @type {number}
     */
    this.parentContainer = -1;
    /**
     * How thick should the drawn line be?
     * @public
     * @instance
     * @type {number}
     */
    this.lineWidth = -1;
    this._class = "RegionLine";
}
module.exports.RegionLine = RegionLine;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing information about a
 * digitizer's physical screen.
 * @tutorial 04 - Handling Pen Events
 * @tutorial 05 - Modifying the Screen
 * @constructor
 */
function DisplayInfo() {
    "use strict";
    /**
     * The width of the physical screen in pixels.
     * @public
     * @instance
     * @type {number}
     */
    this.width = 0;
    /**
     * The height of the physical screen in pixels.
     * @public
     * @instance
     * @type {number}
     */
    this.height = 0;
    /**
     * The color depth of the physical screen.
     * @public
     * @instance
     * @type {number}
     */
    this.colorDepth = 0;
    /**
     * The width of a character in the device's block font.
     * @public
     * @instance
     * @type {number}
     */
    this.characterWidth = 0;
    /**
     * The height of a character in the device's block font.
     * @public
     * @instance
     * @type {number}
     */
    this.characterHeight = 0;
    /**
     * The number of regions this device supports.
     * @public
     * @instance
     * @type {number}
     */
    this.regionCount = 0;
    /**
     * The maximum length of a button caption.
     * @public
     * @instance
     * @type {number}
     */
    this.captionLength = 0;
    this._class = "DisplayInfo";
}
module.exports.DisplayInfo = DisplayInfo;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing information about a
 * digitizer's screen settings.
 * @constructor
 */
function DisplaySettings() {
    "use strict";
    /**
     * The currently configured brightness of the screen.
     * @public
     * @instance
     * @type {number}
     */
    this.brightness = -1;
    /**
     * The currently configured contrast of the screen.
     * @public
     * @instance
     * @type {number}
     */
    this.contrast = -1;
    /**
     * The current red calibration setting of the screen.
     * @public
     * @instance
     * @type {number}
     */
    this.red = -1;
    /**
     * The current green calibration setting of the screen.
     * @public
     * @instance
     * @type {number}
     */
    this.green = -1;
    /**
     * The current blue calibration setting of the screen.
     * @public
     * @instance
     * @type {number}
     */
    this.blue = -1;
    /**
     * Command to send.
     * @public
     * @instance
     * @type {number}
     */
    this.command = -1;
    this._class = "DisplaySettings";
}
module.exports.DisplaySettings = DisplaySettings;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing information about the
 * digitizer's currently installed software.
 * @constructor
 */
function SoftwareVersion() {
    "use strict";
    /**
     * Stores the current kernel version of the device.
     * @public
     * @instance
     * @type {module:dto~KernelVersion}
     */
    this.kernelVersion = new KernelVersion();
    /**
     * Stores the current firmware version of the device.
     * @public
     * @instance
     * @type {module:dto~FirmwareVersion}
     */
    this.firmwareVersion = new FirmwareVersion();
    /**
     * Stores the signature of the currently installed ASIC in the device.
     * @public
     * @instance
     * @type {number}
     */
    this.asicSignature = -1;
    this._class = "SoftwareVersion";
}
module.exports.SoftwareVersion = SoftwareVersion;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing information regarding the
 * digitizer's current and supported modes.
 * @constructor
 */
function OperatingMode() {
    "use strict";
    /**
     * Lists the operating modes the currently connected device supports.
     * @public
     * @instance
     * @type {module:dto~SupportedOperatingModes}
     */
    this.supportedModes = undefined;
    /**
     * The current acquisition mode. All modes: touch, pen, autoTouch, command,
     * securePen and secureAutoTouch.
     * @public
     * @instance
     * @type {string}
     */
    this.acquisitionMode = "";
    /**
     * The current touch mode. All modes: single, singleLiftoff, stream and
     * streamLiftoff.
     * @public
     * @instance
     * @type {string}
     */
    this.touchMode = "";
    /**
     * The current pen mode. All modes: single, singleLiftoff, stream,
     * streamLiftoff and continuous.
     * @public
     * @instance
     * @type {string}
     */
    this.penMode = "";
    this._class = "OperatingMode";
}
module.exports.OperatingMode = OperatingMode;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type enumerates all possible supported operating modes and
 * lists all that the current device currently supports.
 * @constructor
 */
function SupportedOperatingModes() {
    "use strict";
    /**
     * Does this device support touch mode?
     * @public
     * @instance
     * @type {boolean}
     */
    this.touch = false;
    /**
     * Does this device support pen mode?
     * @public
     * @instance
     * @type {boolean}
     */
    this.pen = false;
    /**
     * Does this device support auto touch mode?
     * @public
     * @instance
     * @type {boolean}
     */
    this.autoTouch = false;
    /**
     * Does this device support secure pen mode?
     * @public
     * @instance
     * @type {boolean}
     */
    this.securePen = false;
    /**
     * Does this device support secure auto touch mode?
     * @public
     * @instance
     * @type {boolean}
     */
    this.secureAutoTouch = false;
    this._class = "SupportedOperatingModes";
}
module.exports.SupportedOperatingModes = SupportedOperatingModes;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents a physical card swipe on a digitizer.
 * @tutorial 07 - Handling Card Swipes
 * @constructor
 */
function CardSwipe() {
    "use strict";
    /**
     * The detected format of this card swipe. Currently either: financial,
     * personalid or unknown.
     * @public
     * @instance
     * @type {string}
     */
    this.format = false;
    /**
     * Lists the error that occurred while reading track one (if one occurred).
     * The value will either be false (if no error occurred) or one of the
     * following: "Parity Error(s)", "Unknown Format", "Corrupt Track" or
     * "Unknown Error".
     * @public
     * @instance
     * @type {string}
     */
    this.trackOneError = false;
    /**
     * Lists the error that occurred while reading track two (if one occurred).
     * The value will either be false (if no error occurred) or one of the
     * following: "Parity Error(s)", "Unknown Format", "Corrupt Track" or
     * "Unknown Error".
     * @public
     * @instance
     * @type {string}
     */
    this.trackTwoError = false;
    /**
     * Lists the error that occurred while reading track three (if one occurred).
     * The value will either be false (if no error occurred) or one of the
     * following: "Parity Error(s)", "Unknown Format", "Corrupt Track" or
     * "Unknown Error".
     * @public
     * @instance
     * @type {string}
     */
    this.trackThreeError = false;
    /**
     * The data contained in track one if it was successfully read.
     * @public
     * @instance
     * @type {Array<number>}
     */
    this.trackOne = false;
    /**
     * The data contained in track two if it was successfully read.
     * @public
     * @instance
     * @type {Array<number>}
     */
    this.trackTwo = false;
    /**
     * The data contained in track three if it was successfully read.
     * @public
     * @instance
     * @type {Array<number>}
     */
    this.trackThree = false;
    this._class = "CardSwipe";
}
module.exports.CardSwipe = CardSwipe;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents a connection attempt to an already leased device.
 * @constructor
 */
function DeviceContention() {
    "use strict";
     /**
     * The UUID of the device. This uniquely identifies the device.
     * @public
     * @instance
     * @type {string}
     */
    this.uuid = "";
    this._class = "DeviceContention";
}
module.exports.DeviceContention = DeviceContention;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents a pen-down event on the glass of a digitizer.
 * @tutorial 04 - Handling Pen Events
 * @constructor
 */
function PenDown() {
    "use strict";
    /**
     * The x coordinate of the pen down event as absolute position in pixels.
     * Subpixel resolution is provided where available.
     * @public
     * @instance
     * @type {number}
     */
    this.x = -1;
    /**
     * The y coordinate of the pen down event as absolute position in pixels.
     * Subpixel resolution is provided where available.
     * @public
     * @instance
     * @type {number}
     */
    this.y = -1;
    /**
     * This is the timestamp on which the pen hit the glass.
     * @public
     * @instance
     * @type {number}
     */
    this.time = Date.now();
    this._class = "PenDown";
}
module.exports.PenDown = PenDown;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents a pen-up event on the glass of a digitizer.
 * @tutorial 04 - Handling Pen Events
 * @constructor
 */
function PenUp() {
    "use strict";
    /**
     * The x coordinate of the pen down event as absolute position in pixels.
     * Subpixel resolution is provided where available.
     * @public
     * @instance
     * @type {number}
     */
    this.x = -1;
    /**
     * The y coordinate of the pen down event as absolute position in pixels.
     * Subpixel resolution is provided where available.
     * @public
     * @instance
     * @type {number}
     */
    this.y = -1;
    /**
     * This is the timestamp on which the pen hit the glass.
     * @public
     * @instance
     * @type {number}
     */
    this.time = Date.now();
    this._class = "PenUp";
}
module.exports.PenUp = PenUp;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents a pen-move event on the glass of a digitizer.
 * @tutorial 04 - Handling Pen Events
 * @constructor
 */
function PenMove() {
    "use strict";
    /**
     * The x coordinate of the pen down event as absolute position in pixels.
     * Subpixel resolution is provided where available.
     * @public
     * @instance
     * @type {number}
     */
    this.x = -1;
    /**
     * The y coordinate of the pen down event as absolute position in pixels.
     * Subpixel resolution is provided where available.
     * @public
     * @instance
     * @type {number}
     */
    this.y = -1;
    /**
     * This is the timestamp on which the pen hit the glass.
     * @public
     * @instance
     * @type {number}
     */
    this.time = Date.now();
    this._class = "PenMove";
}
module.exports.PenMove = PenMove;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents a button down event on the glass of a
 * digitizer.
 * @tutorial 04 - Handling Pen Events
 * @constructor
 */
function ButtonDown() {
    "use strict";
    /**
     * The region id of the button being depressed.
     * @public
     * @instance
     * @type {number}
     */
    this.id = -1;
    /**
     * The label of the button being depressed.
     * @public
     * @instance
     * @type {string}
     */
    this.label = "";
    this._class = "ButtonDown";
}
module.exports.ButtonDown = ButtonDown;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents a button up event on the glass of a digitizer.
 * This happens regardless of whether or not the button actually gets pressed (
 * for instance if someone draws off the button).
 * @tutorial 04 - Handling Pen Events
 * @constructor
 */
function ButtonUp() {
    "use strict";
    /**
     * The region id of the button no longer being depressed.
     * @public
     * @instance
     * @type {number}
     */
    this.id = -1;
    /**
     * The label of the button no longer being depressed.
     * @public
     * @instance
     * @type {string}
     */
    this.label = "";
    this._class = "ButtonUp";
}
module.exports.ButtonUp = ButtonUp;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents a button press event on the glass of a digitizer.
 * @tutorial 04 - Handling Pen Events
 * @constructor
 */
function ButtonPress() {
    "use strict";
   /**
     * The region id of the button that was pressed.
     * @public
     * @instance
     * @type {number}
     */
    this.id = -1;
    /**
     * The label of the button that was pressed.
     * @public
     * @instance
     * @type {string}
     */
    this.label = "";
    /**
     * Was the screen cleared as a result of this press?
     * @public
     * @instance
     * @type {boolean}
     */
    this.clear = false;
    /**
     * Should the application consider the signature "accepted"?
     * @public
     * @instance
     * @type {boolean}
     */
    this.accept = false;
    this._class = "ButtonPress";
}
module.exports.ButtonPress = ButtonPress;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents a rendered image of a signature generated by a digitizer.
 * @tutorial 03 - Getting a Signature Image
 * @constructor
 */
function RenderedImage() {
    "use strict";
    /**
     * The mime type of the image being returned.
     * @public
     * @instance
     * @type {string}
     */
    this.type = "image/png";
    /**
     * The width in pixels of the rendered image.
     * @public
     * @instance
     * @type {number}
     */
    this.width = -1;
    /**
     * The height in pixels of the rendered image.
     * @public
     * @instance
     * @type {number}
     */
    this.height = -1;
    /**
     * A data url containing the encoded image.
     * @public
     * @instance
     * @type {string}
     */
    this.data = "";
    this._class = "RenderedImage";
}
module.exports.RenderedImage = RenderedImage;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This class is used to update the rendering settings on the server.
 * @tutorial 03 - Getting a Signature Image
 * @constructor
 */
function RenderSettingsUpdateRequest() {
    "use strict";
    /**
     * The render settings to send to the server.
     * @public
     * @instance
     * @type {module:dto~RenderSettings}
     */
    this.renderSettings = undefined;
    /**
     * Correlation Id to send with the call.
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "RenderSettingsUpdateRequest";
}
module.exports.RenderSettingsUpdateRequest = RenderSettingsUpdateRequest;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This class is used to receive the render settings as interpreted
 * by the server.
 * @tutorial 03 - Getting a Signature Image
 * @constructor
 */
function RenderSettingsUpdateResponse() {
    "use strict";
    /**
     * The render settings currently in use by the server.
     * @public
     * @instance
     * @type {module:dto~RenderSettings}
     */
    this.renderSettings = undefined;
    /**
     * Correlation Id that was sent with the request to update.
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "RenderSettingsUpdateResponse";
}
module.exports.RenderSettingsUpdateResponse = RenderSettingsUpdateResponse;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents the rendering settings of a device used to
 * create RenderedImages.
 * @tutorial 03 - Getting a Signature Image
 * @constructor
 */
function RenderSettings() {
    "use strict";
    /**
     * The mime type of the image to generate.
     * @public
     * @instance
     * @type {string}
     */
    this.type = "image/png";
    /**
     * This controls how the signature is actually rendered.
     * @public
     * @instance
     * @type {module:dto~PenStyle}
     */
    this.penStyle = undefined;
    /**
     * What scaling should be applied to the resulting image.
     * @public
     * @instance
     * @type {number}
     */
    this.scale = 3;
    /**
     * Specifies whether or not the resulting image will be cropped.
     * @public
     * @instance
     * @type {boolean}
     */
    this.crop = false;
    this._class = "RenderSettings";
}
module.exports.RenderSettings = RenderSettings;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents an OmniScript instance on the network.
 * @constructor
 */
function Instance() {
    "use strict";
    /**
     * A list of device UUID's present on this system.
     * @public
     * @instance
     * @type {Array<string>}
     */
    this.devices = [];
    /**
     * The port which the plaintext web socket is being hosted. Value should be
     * -1 if no plaintext websocket server is running.
     * @public
     * @instance
     * @type {number}
     */
    this.socketPort = 0;
    /**
     * The port which the SSL encrypted websocket is being hosted on. Value
     * should be -1 if no encrypted websocket server is running.
     * @public
     * @instance
     * @type {number}
     */
    this.sslSocketPort = 0;
    /**
     * The hostname as returned by the operating system of the server hosting
     * OmniScript.
     * @public
     * @instance
     * @type {string}
     */
    this.hostName = "";
    this._class = "Instance";
}
module.exports.Instance = Instance;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents a OmniScript instance shutting down on the network.
 * @constructor
 */
function InstanceShutdown() {
    "use strict";
    /**
     * The hostname as returned by the operating system of the server hosting
     * OmniScript.
     * @public
     * @instance
     * @type {string}
     */
    this.hostName = "";
    this._class = "InstanceShutdown";
}
module.exports.InstanceShutdown = InstanceShutdown;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents a request to enumerate OmniScript devices on the
 * network.
 * @constructor
 */
function Enumerate() {
    "use strict";
    /**
     * A UUID to search for. False if all servers should be returned.
     * @public
     * @instance
     * @type {string}
     */
    this.uuid = false;
    this._class = "Enumerate";
}
module.exports.Enumerate = Enumerate;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents a physical device plugged into a system.
 * @constructor
 */
function DeviceEndpoint() {
    "use strict";
    /**
     * The UUID of the device. This uniquely identifies the device.
     * @public
     * @instance
     * @type {string}
     */
    this.uuid = "";
    /**
     * The operating system specific path associated with this device. Should be
     * unique per server.
     * @public
     * @instance
     * @type {string}
     */
    this.path = "";
    /**
     * The vendor id of the device, hex encoded.
     * @public
     * @instance
     * @type {string}
     */
    this.vendorId = "";
    /**
     * The product id of the device, hex encoded.
     * @public
     * @instance
     * @type {string}
     */
    this.productId = "";
    /**
     * The manufacturer name of the device according to the operating system.
     * @public
     * @instance
     * @type {string}
     */
    this.manufacturer = "";
    /**
     * The product name of the device according to the operating system.
     * @public
     * @instance
     * @type {string}
     */
    this.product = "";
    this._class = "DeviceEndpoint";
}
module.exports.DeviceEndpoint = DeviceEndpoint;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents a list of devices hosted on a particular
 * server.
 * @tutorial 02 - Connecting
 * @constructor
 */
function DeviceEnumeration() {
    "use strict";
    /**
     * A list of devices hosted on the local server.
     * @public
     * @instance
     * @type {Array<module:dto~DeviceEndpoint>}
     */
    this.devices = false;
    /**
     * The operating system's host name.
     * @public
     * @instance
     * @type {string}
     */
    this.hostName = "";
    /**
     * Indicates whether or not this OmniScript instance is running
     * in safe mode.
     * @public
     * @instance
     * @type {boolean}
     */
    this.safe = true;
    this._class = "DeviceEnumeration";
}
module.exports.DeviceEnumeration = DeviceEnumeration;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This class is sent when a connection is first established to the
 * web socket.
 * @tutorial 02 - Connecting
 * @constructor
 */
function ConnectionOpen() {
    "use strict";
    /**
     * This contains a set of information about the server.
     * @public
     * @instance
     * @type {module:dto~DeviceEnumeration}
     */
    this.serverInfo = undefined;
    this._class = "ConnectionOpen";
}
module.exports.ConnectionOpen = ConnectionOpen;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This message is sent over the wire when the device membership of
 * the host changes.
 * @tutorial 08 - Handling Membership Changes
 * @constructor
 */
function ConnectionDeviceMembershipChange() {
    "use strict";
    /**
     * This contains a set of devices currently attached to the server.
     * @public
     * @instance
     * @type {module:dto~DeviceEnumeration}
     */
    this.devices = undefined;
    this._class = "ConnectionDeviceMembershipChange";
}
module.exports.ConnectionDeviceMembershipChange = ConnectionDeviceMembershipChange;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents a firmware update request.
 * @constructor
 */
function FirmwareUpdateRequest() {
    "use strict";
    /**
     * The firmware file to update the device with encoded as a data url.
     * @public
     * @instance
     * @type {string}
     */
    this.update = undefined;
    /**
     * The correlation id to send with the request.
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "FirmwareUpdateRequest";
}
module.exports.FirmwareUpdateRequest = FirmwareUpdateRequest;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is sent to the client after firmware has been
 * successfully sent to the device.
 * @constructor
 */
function FirmwareUpdateResponse() {
    "use strict";
    /**
     * The correlation id sent with the request.
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "FirmwareUpdateResponse";
}
module.exports.FirmwareUpdateResponse = FirmwareUpdateResponse;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This class represents a progress update to a client regarding new
 * firmware being programmed to their device.
 * @constructor
 * @param {number} percent Percentage of the firmware programmed.
 * @param {string} cid Correlation id.
 */
function FirmwareUpdateNotice(percent, cid) {
    "use strict";
    /**
     * Percentage (between 0 and 1) of the firmware currently copied to the
     * device.
     * @public
     * @instance
     * @type {number}
     */
    this.percent = percent;
    /**
     * The correlation id sent with the request.
     * @public
     * @instance
     * @type {string}
     */
    this.cid = cid;
    this._class = "FirmwareUpdateNotice";
}
module.exports.FirmwareUpdateNotice = FirmwareUpdateNotice;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This class represents an exception being thrown that should be 
 * handled by any remote application.
 * @constructor
 * @param {string} message Message to pass along regarding the cause of the exception.
 */
function ScriptelException(message, cid) {
    "use strict";
    /**
     * Message to return to client.
     * @public
     * @instance
     * @type {string}
     */
    this.message = message;
    /**
     * Correlation id of the call that generated this exception (if available).
     * @public
     * @instance
     * @type {string}
     */
    this.cid = cid;
    this._class = "ScriptelException";
}
module.exports.ScriptelException = ScriptelException;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This class when sent from the client will attempt to close the
 * connection to their device.
 * @constructor
 */
function DeviceCloseRequest() {
    "use strict";
    /**
     * Correlation id to pass through.
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "DeviceCloseRequest";
}
module.exports.DeviceCloseRequest = DeviceCloseRequest;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This class indicates to client that their device connection has
 * been severed.
 * @tutorial 08 - Handling Membership Changes
 * @constructor
 */
function DeviceCloseResponse() {
    "use strict";
    /**
     * Correlation id of the call that generated this event (if available).
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    /**
     * If available this will contain the reason why the connection was severed.
     * @public
     * @instance
     * @type {string}
     */
    this.reason = "";
    this._class = "DeviceCloseResponse";
}
module.exports.DeviceCloseResponse = DeviceCloseResponse;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This class indicates that the client wishes to open a device.
 * @tutorial 02 - Connecting
 * @constructor
 */
function DeviceOpenRequest() {
    "use strict";
    /**
     * The UUID of the device the client wishes to open.
     * @public
     * @instance
     * @type {string}
     */
    this.uuid = "";
    /**
     * An optional correlation id that will be returned in the response.
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "DeviceOpenRequest";
}
module.exports.DeviceOpenRequest = DeviceOpenRequest;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is returned to the client after a DeviceOpenRequest
 * indicating that the request to open the device was successful.
 * @tutorial 02 - Connecting
 * @tutorial 03 - Getting a Signature Image
 * @tutorial 04 - Handling Pen Events
 * @tutorial 05 - Modifying the Screen
 * @tutorial 06 - Forcing Calibration
 * @tutorial 08 - Handling Membership Changes
 * @constructor
 */
function DeviceOpenResponse() {
    "use strict";
    /**
     * All information that has been gathered about the device.
     * @public
     * @instance
     * @type {module:dto~ScriptelDevice}
     */
    this.device = undefined;
    /**
     * This will be set to the correlation id contained in the request (if set).
     * @instance
     * @public
     * @type {string}
     */
    this.cid = "";
    this._class = "DeviceOpenResponse";
}
module.exports.DeviceOpenResponse = DeviceOpenResponse;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This class indicates to a client that the device's calibration
 * state has been updated.
 * @constructor
 * @tutorial 06 - Forcing Calibration
 * @param {string|boolean} point Point being waited for.
 * @param {boolean} done Session complete?
 */
function CalibrationUpdate(point, done) {
    "use strict";
    /**
     * The point which is currently being waited for as part of a calibration
     * session.
     * @public
     * @instance
     * @type {string}
     */
    this.point = point;
    /**
     * Has calibration finished?
     * @public
     * @instance
     * @type {boolean}
     */
    this.done = done;
    this._class = "CalibrationUpdate";
}
module.exports.CalibrationUpdate = CalibrationUpdate;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This class stores the error correction map for this device.
 * @constructor
 */
function ScreenErrorCorrection() {
	"use strict";
    /**
	 * The xMin of the screen error correction.
	 * @public
	 * @instance
	 * @type {number}
	 */
    this.xMin = -1;
	/**
	 * The xDelta of the screen error correction.
	 * @public
	 * @instance
	 * @type {number}
	 */
    this.xDelta = -1;
	/**
	 * The yMin of the screen error correction.
	 * @public
	 * @instance
	 * @type {number}
	 */
    this.yMin = -1;
	/**
	 * The yDelta of the screen error correction.
	 * @public
	 * @instance
	 * @type {number}
	 */
    this.yDelta = -1;
	/**
	 * The xOffset of the screen error correction.
	 * @public
	 * @instance
	 * @type {number}
	 */
    this.xOffset = -1;
	/**
	 * The yOffset of the screen error correction.
	 * @public
	 * @instance
	 * @type {number}
	 */
    this.yOffset = -1;

	/**
	 * The boundX1 of the screen error correction.
	 * @public
	 * @instance
	 * @type {number}
	 */
    this.boundX1 = 0xFFFFFFFF;
	/**
	 * The boundY1 of the screen error correction.
	 * @public
	 * @instance
	 * @type {number}
	 */
    this.boundY1 = 0xFFFFFFFF;
	/**
	 * The boundX2 of the screen error correction.
	 * @public
	 * @instance
	 * @type {number}
	 */
    this.boundX2 = -1;
	/**
	 * The boundY2 of the screen error correction.
	 * @public
	 * @instance
	 * @type {number}
	 */
    this.boundY2 = -1;

    /**
     * This houses the error correction map stored within the device. There are
     * two indexes in this array that contain arrays of x and y coordinates
     * respectively.
     * @public
     * @instance
     * @type {Array<Array<number>>}
     */
    this.correctionMap = [ [], [] ];

    /**
     * Determines whether or not the error correction table built into the
     * device is valid. If the table isn't valid the device will not start
     * properly.
     * @public
     * @instance
     * @type {boolean}
     */
    this.valid = true;
    this._class = "ScreenErrorCorrection";
}
module.exports.ScreenErrorCorrection = ScreenErrorCorrection;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This class represents a request from the client to the server to
 * update a particular region on the device.
 * @tutorial 05 - Modifying the Screen
 * @constructor
 */
function RegionUpdateRequest() {
    "use strict";
    /**
     * The region to update.
     * @public
     * @instance
     * @type {module:dto~Region}
     */
    this.region = undefined;
    this._class = "RegionUpdateRequest";
}
module.exports.RegionUpdateRequest = RegionUpdateRequest;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This class is sent when a region is successfully written to the
 * device.
 * @tutorial 05 - Modifying the Screen
 * @constructor
 */
function RegionUpdateResponse() {
    "use strict";
    /**
     * The region that was sent to the device.
     * @public
     * @instance
     * @type {module:dto~Region}
     */
    this.region = undefined;
    this._class = "RegionUpdateResponse";
}
module.exports.RegionUpdateResponse = RegionUpdateResponse;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type represents the pen style that will be used to render the
 * signature.
 * @tutorial 03 - Getting a Signature Image
 * @constructor
 */
function PenStyle() {
    "use strict";
    /**
     * The name of the function that will be used to render the signature.
     * Right now the valid options are: ChiselPenStyle, InkwellPenStyle, PlainPenStyle.
     * @public
     * @instance
     * @type {string}
     */
    this.renderFunction = undefined;
    /**
     * Options that will be passed to the renderer. For example strokeStyle
     * lets you set the color of the stroke.
     * @public
     * @instance
     * @type {object<string,string>}
     */
    this.options = {};
}
module.exports.PenStyle = PenStyle;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc Receiving this type indicates to the server that it should force
 * the currently connected device into recalibration mode.
 * @tutorial 06 - Forcing Calibration
 * @constructor
 */
function ForceRecalibrate() {
    this._class = "ForceRecalibrate";
}
module.exports.ForceRecalibrate = ForceRecalibrate;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc Receiving this type indicates to the server that it should push
 * the requested bitmap to the device.
 * @constructor
 */
function PushBitmapRequest() {
    /**
     * The horizontal coordinate for the top-left corner of the bitmap.
     * @public
     * @instance
     * @type {number}
     */
    this.x = -1;
    /**
     * The vertical coordinate for the top-left corner of the bitmap.
     * @public
     * @instance
     * @type {number}
     */
    this.y = -1;
    /**
     * A data URL encoded bitmap image in PNG format.
     * @public
     * @instance
     * @type {string}
     */
    this.data = "";
    /**
     * The bit depth the image should be transferred and rendered in. Supported
     * modes are 1 (black and white) and 4 (gray).
     * @public
     * @instance
     * @type {number}
     */
    this.bitDepth = 4;
    /**
     * Correlation ID, sent back with the response.
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "PushBitmapRequest";
}
module.exports.PushBitmapRequest = PushBitmapRequest;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This is sent to the client to indicates their bitmap has been
 * drawn on the device.
 * @constructor
 */
function PushBitmapResponse() {
    /**
     * Correlation ID
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "PushBitmapResponse";
}
module.exports.PushBitmapResponse = PushBitmapResponse;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This is sent from the client, causes all regions on the screen to be hidden.
 * @constructor
 */
function ClearAllRegionsRequest() {
    /**
     * Correlation ID
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    /**
     * Determines whether the parent container will allow or disallow ink on the screen
     * after it's been cleared.
     * @public
     * @instance
     * @type {boolean}
     */
    this.inkEnabled = true;
    this._class = "ClearAllRegionsRequest";
}
module.exports.ClearAllRegionsRequest = ClearAllRegionsRequest;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This is sent to the client after a request indicating the request has completed.
 * @constructor
 */
function ClearAllRegionsResponse() {
    /**
     * Correlation ID
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "ClearAllRegionsResponse";
}
module.exports.ClearAllRegionsResponse = ClearAllRegionsResponse;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This is sent from the client, causes the screen to be cleared and reset. Regions are redrawn.
 * @constructor
 */
function ClearScreenRequest() {
    /**
     * Correlation ID
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "ClearScreenRequest";
}
module.exports.ClearScreenRequest = ClearScreenRequest;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This is sent to the client after a request indicating the request has completed.
 * @constructor
 */
function ClearScreenResponse() {
    /**
     * Correlation ID
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "ClearScreenResponse";
}
module.exports.ClearScreenResponse = ClearScreenResponse;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This request indiciates that OmniScript should reset the device to it's default region configuration.
 * @constructor
 */
function ResetDefaultRegionsRequest() {
    /**
     * Correlation ID
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "ResetDefaultRegionsRequest";
}
module.exports.ResetDefaultRegionsRequest = ResetDefaultRegionsRequest;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This is sent to the client after a request indicating the request has completed.
 * @constructor
 */
function ResetDefaultRegionsResponse() {
    /**
     * Correlation ID
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "ResetDefaultRegionsResponse";
}
module.exports.ResetDefaultRegionsResponse = ResetDefaultRegionsResponse;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This request tells OmniScript to store the current region configuration for later recall. Only persists for the lifetime of the device lease.
 * @constructor
 */
function SaveRegionPresetRequest() {
    /**
     * Correlation ID
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    /**
     * The name of the preset to identify it later.
     * @public
     * @instance
     * @type {string}
     */
    this.name = "";
    this._class = "SaveRegionPresetRequest";
}
module.exports.SaveRegionPresetRequest = SaveRegionPresetRequest;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This is sent to the client after a request indicating the request has completed.
 * @constructor
 */
function SaveRegionPresetResponse() {
    /**
     * Correlation ID
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "SaveRegionPresetResponse";
}
module.exports.SaveRegionPresetResponse = SaveRegionPresetResponse;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This request tells OmniScript to recall the named region preset.
 * @constructor
 */
function RecallRegionPresetRequest() {
    /**
     * Correlation ID
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    /**
     * The name of the preset to recall.
     * @public
     * @instance
     * @type {string}
     */
    this.name = "";
    this._class = "RecallRegionPresetRequest";
}
module.exports.RecallRegionPresetRequest = RecallRegionPresetRequest;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This is sent to the client after a request indicating the request has completed.
 * @constructor
 */
function RecallRegionPresetResponse() {
    /**
     * Correlation ID
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "RecallRegionPresetResponse";
}
module.exports.RecallRegionPresetResponse = RecallRegionPresetResponse;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This class requests that the serial number of a device be changed.
 * this request type is not normally usable.
 * @constructor
 */
function SetSerialNumberRequest() {
    /**
     * Correlation ID
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    /**
     * Serial number to set.
     * @public
     * @instance
     * @type {string}
     */
    this.serialNumber = "";
    this._class = "SetSerialNumberRequest";
}
module.exports.SetSerialNumberRequest = SetSerialNumberRequest;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc Response to the serial number update.
 * @constructor
 */
function SetSerialNumberResponse() {
    /**
     * Correlation ID
     * @public
     * @instance
     * @type {string}
     */
    this.cid = "";
    this._class = "SetSerialNumberResponse";
}
module.exports.SetSerialNumberResponse = SetSerialNumberResponse;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc Request to update the display settings of a device.
 * @constructor
 */
function UpdateDisplaySettingsRequest() {
    /**
     * Brightness level of the backlight to set.
     * @public
     * @instance
     * @type {number}
     */
    this.brightness = 0;
    /**
     * Constrast level of the screen to set.
     * @public
     * @instance
     * @type {number}
     */
    this.contrast = 0;
    /**
     * Red level of the screen to set.
     * @public
     * @instance
     * @type {number}
     */
    this.red = 0;
    /**
     * Green level of the screen to set.
     * @public
     * @instance
     * @type {number}
     */
    this.green = 0;
    /**
     * Blue level of the screen to set.
     * @public
     * @instance
     * @type {number}
     */
    this.blue = 0;
    this._class = "UpdateDisplaySettingsRequest";
}
module.exports.UpdateDisplaySettingsRequest = UpdateDisplaySettingsRequest;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc Response to request to update the display settings of a device.
 * @constructor
 */
function UpdateDisplaySettingsResponse() {
    this._class = "UpdateDisplaySettingsResponse";
}
module.exports.UpdateDisplaySettingsResponse = UpdateDisplaySettingsResponse;
