var fs     = require("fs");
var zlib   = require("zlib");
var crypto = require("crypto");
var config = JSON.parse(fs.readFileSync("lib/scriptel-config.json"));

function FirmwareUpdate(buffer, lease, config) {
    "use strict";
    this.buffer = buffer;
    this.lease = lease;
    this.config = config;
    this.signatureValid = false;
    this.firmware = false;
    this.updateCallback = false;

    this.compressed = (buffer.length > 2 && buffer.readUInt8(0) === 0x1f && buffer.readUInt8(1) === 0x8b);
}
FirmwareUpdate.prototype.registerLoadCallback = function (f) {
    "use strict";
    this.updateCallback = f;
};
FirmwareUpdate.prototype.loadFirmware = function (errorCallback) {
    "use strict";
    var t = this;
    var error = false;
    if (this.compressed) {
        zlib.gunzip(this.buffer, function (err, data) {
            if (err) {
                errorCallback(new Error("There was a problem decompressing the firmware: " + err));
                return;
            }
            t.buffer = data.toString("UTF-8");
            error = t.parseFirmware();
            if(error) {
                errorCallback(error)
            }
        });
    } else {
        this.buffer = this.buffer.toString("UTF-8");
        error = this.parseFirmware();
        if(error) {
            errorCallback(error);
        }
    }
};
FirmwareUpdate.prototype.parseFirmware = function () {
    "use strict";
    //Now we need to attempt to split this into the firmware payload and
    //firmware signature.
    var START_TOKEN = "-----BEGIN SIGNATURE-----\n";
    var END_TOKEN = "-----END SIGNATURE-----\n";
    var sigStart = this.buffer.indexOf(START_TOKEN);
    var sigEnd = this.buffer.indexOf(END_TOKEN);

    if (sigStart >= 0 && sigEnd >= 0) {
        var data = this.buffer.substring(0, sigStart);
        sigStart += START_TOKEN.length;
        var signature = new Buffer(this.buffer.substring(sigStart, sigEnd).replace("\n", ""), 'base64');

        var publicKey = fs.readFileSync(this.config.firmwareupdate.key);
        var verify = crypto.createVerify('RSA-MD5');
        verify.update(new Buffer(data, "UTF-8"));
        this.signatureValid = verify.verify(publicKey, signature);
        if (this.signatureValid) {
            this.firmware = JSON.parse(data);
        } else {
            return new Error("Firmware appears to have been tampered with.");
        }

        if (this.validateDevice()) {
            //Lets do it!
            this.updateFirmware();
        } else {
            return new Error("Uploaded firmware doesn't support this device.");
        }
    } else {
        return new Error("Firmware seems to be missing a signature.");
    }
    return false;
};
FirmwareUpdate.prototype.validateDevice = function () {
    "use strict";

    //If the safety override is set you can do stupid things. Don't do stupid things.
    if(config.safetyOverride) {
        return true;
    }

    //Now we need to match device strings.
    var device = this.lease.deviceInfo;
    var validDevices = this.firmware.devices;
    var x;
    for (x = 0; x < validDevices.length; x++) {
        var vendorPattern = validDevices[x].vendorId.replace("?", ".?");
        var productPattern = validDevices[x].productId.replace("?", ".?");
        if (device.vendorId.match(vendorPattern) && device.productId.match(productPattern)) {
            return true;
        }
    }
    return false;
};
FirmwareUpdate.prototype.updateFirmware = function () {
    "use strict";
    var devHandle = this.lease.deviceHandle;
    var len = this.firmware.firmware.length;
    var x, i;
    for (x = 0; x < len; x++) {
        var update = this.firmware.firmware[x];
        if(update[0] === 25) {
            //FirmwareDownload
            var obj = {"address": 0, "checksum": 0, "data": []};
            obj.address = ((update[4] & 0xFF) << 24) + ((update[3] & 0xFF) << 16) + ((update[2] & 0xFF) << 8) + (update[1] & 0xFF);
            obj.checksum = (update[update.length - 1] & 0xFF);
            for(i = 5; i < update.length - 1; i++) {
                obj.data[i - 5] = update[i];
            }
            devHandle.setFirmwarePage(obj);
        } else if(update[0] === 29) {
            //CommandStatus
            devHandle.setFirmwareComplete();
        }
        
        if (this.updateCallback) {
            this.updateCallback((x + 1) / len);
        }
    }
};
module.exports = FirmwareUpdate;