//NodeJS Requires
var fs                = require("fs");
var Canvas            = require("canvas");
var crypto            = require("crypto");
var os                = require("os");
var proscript         = require("scriptel-proscript");
var Logger            = require("../lib/scriptel-logger.js");
var dto               = require("../lib/scriptel-dto.js");
var FirmwareUpdate    = require("../lib/scriptel-firmware.js");
var ScriptelDiscovery = require("../lib/scriptel-discovery.js");
var ScriptelWebServer = require("../lib/scriptel-webserver.js");
var penstyle          = require("../lib/scriptel-penstyle.js");
var plugins           = require("../lib/scriptel-plugins.js");
var CommandLine       = require("../lib/scriptel-commandline.js");

var osVersion = {
	"versionNumber": "2.1.80",
	"buildDate": "Tue, 01 Nov 2022 18:22:43 +0000",
	"buildNumber": "180",
	"revision": "15921",
	"jobName": "omniscript"
};

var DeviceManager = new proscript.DeviceManager();

//Load Configuration.
var config = JSON.parse(fs.readFileSync("lib/scriptel-config.json"));
CommandLine.parse(process.argv);
CommandLine.override(config);

Logger.setConfiguration(config);
plugins.setConfiguration(config);

Logger.info("Scriptel OmniScript Integrated Web Server Starting");
printVersion();

plugins.setLogger(Logger);
plugins.load();


/**
 * This variable stores the universal protocol driver.
 * @global
 */
var ScripTouchDevice = require("../lib/scriptel-protocol-driver.js");

/**
 * This variable is used to blacklist EasyScript devices since
 * these devices can't provide HID coordinate output making
 * OmniScript pretty useless.
 * @global
 */
var blackList = [
    { "vendorId": 0x18fb, "productId": 0x01c5, "name": "ST1401_STN"  },
    { "vendorId": 0x18fb, "productId": 0x01c9, "name": "ST1476_STN"  },
    { "vendorId": 0x18fb, "productId": 0x01c0, "name": "ST1501_STN"  },
    { "vendorId": 0x18fb, "productId": 0x01c2, "name": "ST1501_PYJ"  },
    { "vendorId": 0x18fb, "productId": 0x01c3, "name": "ST1501_BPYJ" },
    { "vendorId": 0x18fb, "productId": 0x01c4, "name": "ST1501_PUN"  },
    { "vendorId": 0x18fb, "productId": 0x01c1, "name": "ST1526_STN"  },
    { "vendorId": 0x18fb, "productId": 0x01c6, "name": "ST1551_STN"  },
    { "vendorId": 0x18fb, "productId": 0x01ca, "name": "ST1551_PYJ"  },
    { "vendorId": 0x18fb, "productId": 0x01cb, "name": "ST1571_STN"  }
];

/**
 * This variable is responsible for storing coordinate events (and potentially
 * other signature-related meta data) such that a rendered signature can be
 * produced on demand. Stored as a map of device paths to device states.
 * @global
 * @type {Object<string,DeviceState>}
 */
var deviceStates = {};
/**
 * This variable is responsible for storing relationships between connected
 * devices and the connections into the server that are currently leasing (and
 * therefore locking) them.
 * @global
 * @type {Object<string,DeviceLease>}
 */
var leases = {};
/**
 * This variable is responsible for tracking all open connections. Used for 
 * broadcasting global events (such as device membership changes).
 * @global
 * @type {Array<WebSocketConnection>}
 */
var connections = [];

/**
 * This variable is responsible for storing the most current
 * version of the device membership list loaded from our HID
 * driver. Updated by callback.
 * @global
 * @type {Array<module:dto~DeviceEndpoint>}
 */
var devices = reloadDevices();

/**
 * This catches any uncaught exceptions, so we don't crash.
 * If this gets called, something horrible happened.
 */
process.on('uncaughtException', function(err) {
    console.trace("Unhandled exception: ", err);
});

/**
 * This callback is called by our native HID driver when there is a change to
 * the system's device memberships.
 * @param {module:omniscript~DeviceEndpointChange} dev The device endpoint that changed and the event type.
 * @callback DeviceEndpointChangeCallback
 */
var deviceChangeCallback = function (action) {
    "use strict";
    var devPath, x;
    var leasedPaths = [];
    var dev = {"path": action.path, "action": false };
    if(action.type == "ATTACHED") {
        dev.action = "add";
    } else {
        dev.action = "remove";
    }

    Logger.info("Operating system notified us of a device membership change: path={0} action={1}", [dev.path, dev.action]);
    if (dev.action === "remove") {
        //Lets see if we have to break the bad news to anyone.
        for (devPath in leases) {
			if (leases.hasOwnProperty(devPath)) {
				var lease = leases[devPath];
                leasedPaths[leasedPaths.length] = devPath;
				if (lease.deviceInfo.path.toLowerCase() === dev.path.toLowerCase()) {
					Logger.warning("Client currently leasing {0} just lost their physical hardware, foribly disconnecting.", [dev.path]);

					var closed = new dto.DeviceCloseResponse();
					closed.reason = "Device you were connected to has been disconnected from the host.";
                    sendMessage(closed,lease.connection);
					lease.deviceHandle.close();
					delete leases[devPath];
                    delete deviceStates[devPath];
				}
			}
        }
    }

    //Just in case we have an operating system hiccup, lets check to make sure all our devices are
    //still in the list.
    var tmp = reloadDevices();

    var newDevPaths = [];
    for(x=0;x<tmp.length;x++) {
        newDevPaths[newDevPaths.length] = tmp[x].path;
    }

    var found = false, y;
    for(x=0;x<leasedPaths.length;x++) {
        for(y=0;!found && y<newDevPaths.length;y++) {
            if(leasedPaths[x].toLowerCase()===newDevPaths[y].toLowerCase()) {
                found = true;
            }
        }
        if(!found) {
            Logger.warning("Client currently leasing {0} just lost their physical hardware, foribly disconnecting.", [leasedPaths[x]]);

            var closed = new dto.DeviceCloseResponse();
            closed.reason = "Device you were connected to has been disconnected from the host.";
            sendMessage(closed,lease.connection);
            if(leases[leasedPaths[x]] && leases[leasedPaths[x]].deviceHandle) {
                leases[leasedPaths[x]].deviceHandle.close();
            }
            delete leases[leasedPaths[x]];
            delete deviceStates[leasedPaths[x]];
        }
    }

    devices = tmp;

    //Now lets notify everyone their device list is out of date.
    var membershipChange = new dto.ConnectionDeviceMembershipChange();
    membershipChange.devices = getDeviceEnumeration();
    for (x = 0; x < connections.length; x++) {
        sendMessage(membershipChange,connections[x]);
    }
};


//Registers this callback with the native component of OmniScript.
DeviceManager.addDeviceListener(deviceChangeCallback);

/**
 * This variable stores the class responsible for listening for and sending
 * UDP multicast broadcasts across the network.
 * @global
 * @type {ScriptelDiscovery}
 */

var discovery = new ScriptelDiscovery(config);
/**
 * This callback is responsible for managing recieved messages from the
 * multicast group the machine is currently listening to.
 * @param msg The message recieved from the multicast group.
 * @param info Information about who sent the message.
 * @callback DiscoveryMessageCallback
 */
discovery.registerCallback(function (msg, info) {
    "use strict";
    var x;
    if (msg._class === "Enumerate") {
        if (msg.uuid) {
            //They're looking for a specific UUID:
            Logger.info("Discovery member {0}:{1} asked for enumeration for a specific UUID: {2}", [info.address, info.port, msg.uuid]);
            var found = false;
            for (x = 0; x < devices.length && !found; x++) {
                if (devices[x].uuid === msg.uuid) {
                    found = true;
                }
            }
            if (found) {
                discovery.broadcastInstanceInformation(devices);
            }
        } else {
            Logger.info("Discovery member {0}:{1} asked for enumeration of all members.", [info.address, info.port, info]);
            discovery.broadcastInstanceInformation(devices);
        }
    } else if (msg._class === "Instance") {
        msg.address = info.address;
        for (x = 0; x < connections.length; x++) {
            sendMessage(msg, connections[x]);
        }
        Logger.info("Discovery member {0}:{1} notified us about their membership.", [info.address, info.port]);
    } else {
        Logger.info("Discovery member {0}:{1} asked for unknown information: {2}", [info.address, info.port, msg._class]);
    }
});

/**
 * This variable stores the instance of the static web server we're currently
 * hosting. The websocket listener sits on top of the static listener.
 * @global
 * @type {ScriptelWebServer}
 */
var webServer = new ScriptelWebServer(config);
//Associate the web socket handler with our static web server.
webServer.registerWebSocketHandler(webSocketHandler);

/**
 * This function is responsible for handling new WebSocket connection requests.
 * @param {WebSocketConnectionRequest} request Connection request for a WebSocket connection.
 */
function webSocketHandler(request) {
    "use strict";
    var con = request.accept(null, request.origin);
    Logger.info("Accepted a new WebSocket connection from {0} to {1}.", [request.remoteAddress, request.host]);
    connections[connections.length] = con;
    var cOpen = new dto.ConnectionOpen();
    cOpen.serverInfo = getDeviceEnumeration();
    sendMessage(cOpen,con);

    var handleInput = function (msg) {
        var lease = getConnectionLease(con);
        var devState = getConnectionState(con);

        //Lets give the plugins the first crack at this.
        if(plugins.handleInboundMessage(msg,con,devState,lease,handleInput)) {
            return;
        }

        if (msg._class === "DeviceOpenRequest") {
            Logger.fine("Client {0} requested that we open device {1}", [request.remoteAddress, msg.uuid]);
            openDevice(con, msg);

            lease = getConnectionLease(con);
            if(lease && !lease.handleInput) {
                lease.handleInput = handleInput;
            }
        } else if (msg._class === "DeviceCloseRequest") {
            Logger.fine("Client {0} requested that we close their open device.", [request.remoteAddress]);
            if (closeDevice(con)) {
                var closeResponse = new dto.DeviceCloseResponse();
                closeResponse.cid = msg.cid;
                sendMessage(closeResponse,con);
            } else {
                Logger.info("Client {0} requested that we close their device but they don't have one open.", [request.remoteAddress]);
                sendMessage(new dto.ScriptelException("No device currently leased, cannot close device.", msg.cid),con);
            }
        } else if (msg._class === "RenderSettingsUpdateRequest") {
            Logger.fine("Client {0} requested that we update their render settings.", [request.remoteAddress]);
            if (lease) {
                devState.renderSettings = msg.renderSettings;

                var response = new dto.RenderSettingsUpdateResponse();
                response.renderSettings = msg.renderSettings;
                response.cid = msg.cid;
                sendMessage(response,con);
            } else {
                sendMessage(new dto.ScriptelException("No device currently leased, cannot update render settings.", msg.cid), con);
                Logger.warning("Client {0} asked us to update render settings, but hasn't yet opened a device connection.", [request.remoteAddress]);
            }
        } else if (msg._class === "Enumerate") {
            Logger.fine("Client {0} requested that we enumerate OmniScript instances.", [request.remoteAddress]);
            var enumerate = new dto.Enumerate();
            enumerate.uuid = msg.uuid;
            discovery.sendMessage(enumerate);
        } else if (msg._class === "FirmwareUpdateRequest") {
            Logger.fine("Client {0} requested that we perform a firmware update.", [request.remoteAddress]);
            if (lease) {
                var idx = msg.update.indexOf(",");
                var buffer = new Buffer(msg.update.substring(idx + 1), 'base64');
                try {
                    var fwUpdate = new FirmwareUpdate(buffer, lease, config);
                    fwUpdate.registerLoadCallback(function (pct) {
                        sendMessage(new dto.FirmwareUpdateNotice(pct, msg.cid),con);
                    });
                    fwUpdate.loadFirmware(function(err) {
                        if(err) {
                            sendMessage(new dto.ScriptelException("Problem updating firmware: " + err.message, msg.cid),con);
                            Logger.warning("Problem updating firmware for client {0}: {1}", [request.remoteAddress, err]);
                        }
                    });

                    var fwResult = new dto.FirmwareUpdateResponse();
                    fwResult.cid = msg.cid;
                    sendMessage(fwResult,con);
                } catch (e) {
                    sendMessage(new dto.ScriptelException("Problem updating firmware: " + e.message, msg.cid),con);
                    Logger.warning("Problem updating firmware for client {0}: {1}", [request.remoteAddress, e]);
                }
            } else {
                sendMessage(new dto.ScriptelException("No device currently leased, cannot update firmware."),con);
                Logger.warning("Client {0} asked us to update firmware, but hasn't yet opened a device connection.", [request.remoteAddress]);
            }
        } else if (msg._class === "ForceRecalibrate") {
            Logger.fine("Client {0} requested that we perform recalibration.", [request.remoteAddress]);
            if (lease) {
                lease.deviceInfo.protocolDriver.forceRecalibration(lease.deviceHandle);
                var update = new dto.CalibrationUpdate();
                update.point = "Center";
                update.done = false;
                sendMessage(update,con);
                devState.deviceInfo.calibrationMode.calibrating = true;
                devState.deviceInfo.calibrationMode.calibrationPoint = "Center";
                var lastState = devState.deviceInfo.calibrationMode;
                setTimeout(function () {
                    calibrationPoll(con, lease.deviceInfo, lease.deviceHandle, lastState);
                }, 1000);
            } else {
                sendMessage(new dto.ScriptelException("No device currently leased, cannot force calibration."),con);
            }
        } else if (msg._class === "RegionUpdateRequest") {
            Logger.fine("Client {0} requested that we update a region.", [request.remoteAddress]);
            if (lease) {
                if(msg.region!=1) {
                    lease.deviceInfo.protocolDriver.updateRegion(lease.deviceHandle, devState.deviceInfo, msg.region);
                    var r = new dto.RegionUpdateResponse();
                    r.region = msg.region;
                    sendMessage(r,con);
                    devState.deviceInfo.regions[msg.region.id - 1] = msg.region;
                } else {
                    sendMessage(new dto.ScriptelException("The first device region cannot be updated."),con);
                }
            } else {
                sendMessage(new dto.ScriptelException("No device currently leased, cannot update region."),con);
            }
        } else if (msg._class === "PushBitmapRequest") {
            if (lease) {
                var buffer = parseDataUrl(msg.data);
                var imgs = false;
                if(msg.bitDepth===1) {
                    imgs = lease.deviceInfo.protocolDriver.convertBitmapToSpriteBW(buffer);
                } else if(msg.bitDepth===4) {
                    imgs = lease.deviceInfo.protocolDriver.convertBitmapToSpriteGray(buffer);
                }
                
                if(!imgs) {
                    throw "Unknown bit depth: " + msg.bitDepth;
                }
                msg.x = parseInt(msg.x,10);
                msg.y = parseInt(msg.y,10);
                var j;
                for(j = 0; j < imgs.length; j++) {
                    var img = imgs[j];
                    img.x += msg.x;
                    img.y += msg.y;
                    lease.deviceInfo.protocolDriver.sendSprite(lease.deviceHandle, devState.deviceInfo, img);
                }
                
                var r = new dto.PushBitmapResponse();
                if(msg.cid) {
                    r.cid = msg.cid;
                }
                sendMessage(r,con);
            } else {
                sendMessage(new dto.ScriptelException("No device currently leased, cannot push bitmap."),con);
            }
        } else if(msg._class === "ClearScreenRequest") {
            if(lease) {
                lease.deviceInfo.protocolDriver.clearScreen(lease.deviceHandle,devState.deviceInfo);
                var r = new dto.ClearScreenResponse();
                r.cid = msg.cid;
                sendMessage(r,con);
            } else {
                sendMessage(new dto.ScriptelException("No device currently leased, cannot clear screen."),con);
            }
        } else if(msg._class === "ClearAllRegionsRequest") {
            if(lease) {
                var i, region;
                for(i=2;i<devState.deviceInfo.regions.length;i++) {
                    region = {
                        "id": (i+1),
                        "x1": 0,
                        "x2": 1,
                        "y1": 0,
                        "y2": 1,
                        "containerType": {"_class":"RegionContainer"},
                        "flags": {
                            "acceptsTouch": false,
                            "decodeRegion": false,
                            "hasFrame": false,
                            "inUse": false,
                            "visible": false
                        }
                    };
                    lease.deviceInfo.protocolDriver.updateRegion(lease.deviceHandle, devState.deviceInfo, region);
                    devState.deviceInfo.regions[i] = region;
                }
                region = {
                    "id": 2,
                    "x1": 0,
                    "x2": devState.deviceInfo.displayInfo.width-1,
                    "y1": 0,
                    "y2": devState.deviceInfo.displayInfo.height-1,
                    "containerType": {
                        "_class":"RegionContainer",
                        "flags": {
                            "inkEnabled": (msg.inkEnabled)?true:false
                        }
                    },
                    "flags": {
                        "acceptsTouch": true,
                        "decodeRegion": true,
                        "hasFrame": false,
                        "inUse": true,
                        "visible": true
                    }
                };
                lease.deviceInfo.protocolDriver.updateRegion(lease.deviceHandle, devState.deviceInfo, region);
                devState.deviceInfo.regions[1] = region;

                var r = new dto.ClearAllRegionsResponse();
                r.cid = msg.cid;
                sendMessage(r,con);
            } else {
                sendMessage(new dto.ScriptelException("No device currently leased, cannot clear regions."),con);
            }
        } else if(msg._class === "ResetDefaultRegionsRequest") {
            if(lease) {
                var regions = lease.deviceInfo.protocolDriver.resetScreen(lease.deviceHandle, devState.deviceInfo);

                var r = new dto.ResetDefaultRegionsResponse();
                r.cid = msg.cid;
                r.regions = regions;
                sendMessage(r,con);
            } else {
                sendMessage(new dto.ScriptelException("No device currently leased, cannot set default regions."),con);
            }
        } else if(msg._class === "SaveRegionPresetRequest") {
            if(lease) {
                if(msg.name) {
                    lease.regionPresets[msg.name] = JSON.parse(JSON.stringify(devState.deviceInfo.regions));

                    var r = new dto.SaveRegionPresetResponse();
                    r.cid = msg.cid;
                    sendMessage(r,con);
                } else {
                    sendMessage(new dto.ScriptelException("No preset name defined, cannot save preset."),con);    
                }
            } else {
                sendMessage(new dto.ScriptelException("No device currently leased, cannot save preset."),con);
            }
        } else if(msg._class === "RecallRegionPresetRequest") {
            if(lease) {
                if(msg.name) {
                    if(lease.regionPresets[msg.name]) {
                        var regions = lease.regionPresets[msg.name];

                        var i;
                        for(i=1;i<regions.length;i++) {
                            lease.deviceInfo.protocolDriver.updateRegion(lease.deviceHandle, devState.deviceInfo, regions[i]);
                            devState.deviceInfo.regions[i] = regions[i];
                        }

                        var r = new dto.RecallRegionPresetResponse();
                        r.cid = msg.cid;
                        sendMessage(r,con);
                    } else {
                        sendMessage(new dto.ScriptelException("Unable to find preset by requested name."),con);    
                    }
                } else {
                    sendMessage(new dto.ScriptelException("No preset name defined, cannot recall preset."),con);    
                }
            } else {
                sendMessage(new dto.ScriptelException("No device currently leased, cannot recall preset."),con);
            }
        } else if(msg._class === "SetSerialNumberRequest") {
            if(config.safetyOverride) {
                if(lease) {
                    var result = lease.deviceInfo.protocolDriver.setSerialNumber(lease.deviceHandle, devState.deviceInfo, msg.serialNumber);
                    if(result) {
                        sendMessage(new dto.ScriptelException(result.message),con);
                    } else {
                        var r = new dto.SetSerialNumberResponse();
                        r.cid = msg.cid;
                        sendMessage(r,con);
                    }
                } else {
                    sendMessage(new dto.ScriptelException("No device currently leased, cannot set serial number."),con);
                }
            } else {
                sendMessage(new dto.ScriptelException("Unable to update serial numbers when running in normal mode."),con);
            }
        } else if(msg._class === "UpdateDisplaySettingsRequest") {
            if(lease) {
                lease.deviceInfo.protocolDriver.updateDisplaySettings(lease.deviceHandle, devState.deviceInfo, msg);
                var r = new dto.UpdateDisplaySettingsResponse();
                r.cid = msg.cid;
                sendMessage(r,con);
            } else {
                sendMessage(new dto.ScriptelException("No device currently leased, cannot set default regions."),con);
            }
        }
    };

    con.on("message", function(message) {
        if (message.type === "utf8") {
            try {
                var msg = JSON.parse(message.utf8Data);
                handleInput(msg);
            } catch (e) {
                Logger.warning("Invalid data received from client {0}: {1}", [request.remoteAddress, e]);
            }
        }
    });

    con.on("close", function (connection) {
        var devPath, x;
        Logger.info("WebSocket for client {0} disconnected.", [request.remoteAddress]);
        for (devPath in leases) {
			if (leases.hasOwnProperty(devPath)) {
				var lease = leases[devPath];
				if (lease.connection === con) {
					//We can free this lease.
					Logger.info("Client {0} had an active lease for device {1}, closing connection.", [request.remoteAddress, lease.deviceInfo.uuid]);
					lease.deviceHandle.close();
					delete leases[devPath];
					delete deviceStates[devPath];
				}
			}
        }
        for (x = connections.length; x >= 0; x--) {
            if (connection[x] === con) {
                //Splice this out of the array.
                connections.splice(x, 1);
            }
        }
    });
}
/**
 * This function attempts to locate any active leases for a given connection.
 * The first lease found (there should only be one) is returned.
 * @param {WebSocketConnection} con Connection to search for.
 * @return {DeviceLease} Lease if found, false if nothing found.
 */
function getConnectionLease(con) {
    "use strict";
    var devPath;
    for (devPath in leases) {
		if (leases.hasOwnProperty(devPath)) {
			var lease = leases[devPath];
			if (lease.connection === con) {
				return lease;
			}
		}
    }
    return false;
}
function getConnectionState(connection) {
    var lease = getConnectionLease(connection);
    if(lease) {
        return deviceStates[lease.deviceInfo.path];
    }
    return false;
}
/**
 * This function will send a list of all connected devices to a particular
 * WebSocket connection.
 * @param {WebSocketConnection} con WebSocket connection to send to.
 */
function getDeviceEnumeration() {
    "use strict";
    var msg = new dto.DeviceEnumeration();
    msg.devices = devices;
    msg.hostName = os.hostname();
    msg.safe = (config.safetyOverride)?false:true;
    return msg;
}
/**
 * This function will attempt to close a device (and release any leases) based
 * on connection.
 * @param {WebSocketConnection} con Connection to clean up after.
 * @return {boolean} True if a device lease was cleaned up, false otherwise.
 */
function closeDevice(con) {
    "use strict";
    var devPath;
    for (devPath in leases) {
		if (leases.hasOwnProperty(devPath)) {
			var lease = leases[devPath];
			if (lease.connection === con) {
				lease.deviceHandle.close();
				delete leases[devPath];
				delete deviceStates[devPath];
				return true;
			}
		}
    }
    return false;
}
/**
 * This function will attempt to open a device and take out a lease based on its
 * UUID and the web socket connection requesting it.
 * @param {WebSocketConnection} con Connection leasing the device.
 * @param {DeviceOpenRequest} uuid UUID of the device to lease.
 * @return {boolean} True if the device was successfully leased, false otherwise.
 */
function openDevice(con, request) {
    "use strict";
    var devInfo = false;
	var lease, x, devPath;
    for (x = 0; x < devices.length && !devInfo; x++) {
        if (devices[x].uuid.toLowerCase() === request.uuid.toLowerCase()) {
            devInfo = devices[x];
        }
    }
    if (!devInfo) {
        Logger.warning("Client asked us to open a device we don't have a record of: {0}", [request.uuid]);
        sendMessage(new dto.ScriptelException("Unknown device id: " + request.uuid, request.cid),con);
        return false;
    }
    if (leases[devInfo.path]) {
        Logger.warning("Client asked us to open a device that was already leased to another client: {0} ({1})", [request.uuid, devInfo.path]);
        sendMessage(new dto.ScriptelException("Requested device is already opened by another client.", request.cid),con);
        var contention;
        contention = new dto.DeviceContention();
        contention.uuid = leases[devInfo.path].deviceInfo.uuid;
        sendMessage(contention, leases[devInfo.path].connection);
        return false;
    }
    if (!devInfo.protocolDriver) {
        Logger.warning("Client asked us to open a device we don't have a registered protocol driver for: {0} ({1})", [request.uuid, devInfo.path]);
        sendMessage(new dto.ScriptelException("No protocol driver exists for this device.", request.cid),con);
    }
    for (devPath in leases) {
        if (leases.hasOwnProperty(devPath)) {
            lease = leases[devPath];
            if (lease.connection === con) {
                Logger.warning("Client already has a device open ({0}) can't open another on this socket.", [lease.deviceInfo.path]);
                sendMessage(new dto.ScriptelException("Device already open for this socket, please either close the current device or open a new socket.", request.cid),con);
                return false;
            }
        }
    }

    var device = devInfo.devHandle;

    device.open();
    lease = new DeviceLease();
    lease.connection = con;
    lease.deviceInfo = devInfo;
    lease.deviceHandle = device;

    leases[devInfo.path] = lease;

    var retry;
    var hwInfo;
    for(retry = 0; retry < 5; retry++) {
        try {
            hwInfo = devInfo.protocolDriver.getDeviceInfo(device);
            retry = 6;
        } catch(e) {
            Logger.info("Ran into trouble getting device information, retry {0} of 5: {1}", [retry + 1, e]);
        }
    }
    hwInfo.uuid = request.uuid;

    var devState = new DeviceState();
    devState.deviceInfo = hwInfo;
    devState.connection = con;
    devState.renderSettings = new dto.RenderSettings();
    deviceStates[devInfo.path] = devState;

    device.addInputListener(function (r) {
        var retr = devInfo.protocolDriver.readCallback(r, hwInfo);
        if (retr) {
            //If this is a pen event, lets cache the object.
            if (retr._class && retr._class === "Coordinate") {
                retr = manageCoordinateEvent(devState, retr);
            } else if (retr._class && retr._class === "ButtonPress") {
                if(retr.accept) {
                    sendSignature(devState);
                }
                if(retr.clear) {
                    devState.penBuffer = [];
                }
            }

            if (retr) {
                sendMessage(retr,con);
            }
        }
    });

    if (hwInfo.calibrationMode && hwInfo.calibrationMode.calibrating) {
        var lastState = hwInfo.calibrationMode;
        setTimeout(function () {
            calibrationPoll(con, devInfo, device, lastState);
        }, 1000);
    }

    //Let the client know their device is now ready to go.
    var response = new dto.DeviceOpenResponse();
    response.device = hwInfo;
    response.cid = request.cid;
    sendMessage(response,con);

    return true;
}
function manageCoordinateEvent(devState, c) {
    var evt;
    if (!devState.penDown) {
        devState.penDown = true;
        evt = new dto.PenDown();
    } else {
        if (c.flags.penSwitchOn) {
            evt = new dto.PenMove();
        } else {
            devState.penDown = false;
            evt = new dto.PenUp();
        }
    }
    evt.x = c.x * devState.deviceInfo.displayInfo.width;
    evt.y = c.y * devState.deviceInfo.displayInfo.height;
    devState.penBuffer[devState.penBuffer.length] = evt;
    return evt;
}
function calibrationPoll(con, devInfo, device, lastState) {
    "use strict";
    try {
        var retr = devInfo.protocolDriver.getCalibrationData(device);
        if (lastState.calibrating) {
            if (lastState.calibrationPoint !== retr.calibrationPoint) {
                sendMessage(new dto.CalibrationUpdate(retr.calibrationPoint, false),con);
                lastState = retr;
            }
            setTimeout(function () {
                calibrationPoll(con, devInfo, device, lastState);
            }, 1000);
        } else {
            sendMessage(new dto.CalibrationUpdate(false, true),con);
        }
    } catch (e) {
        Logger.warning("Problem polling for calibration updates: {0}", [e]);
    }
}
/**
 * This function attempts to render a signature based off the stored device
 * state. The rendered signature is then sent over the wire as a data URL
 * encapsulated within a RenderedImage object.
 * @param {DeviceState} devState Device State to render.
 */
function sendSignature(devState) {
    "use strict";
    //Fill in Pen Style, since we can't pass functions through JSON.
    if (!devState.renderSettings.penStyle) {
        devState.renderSettings.penStyle = new dto.PenStyle();
        devState.renderSettings.penStyle.renderFunction = penstyle.PlainPenStyle;
        devState.renderSettings.penStyle.options = {};
    } else if (typeof devState.renderSettings.penStyle.renderFunction === "string") {
        if (typeof penstyle[devState.renderSettings.penStyle.renderFunction] === "function") {
            devState.renderSettings.penStyle.renderFunction = penstyle[devState.renderSettings.penStyle.renderFunction];
        } else {
            devState.renderSettings.penStyle.renderFunction = penstyle.PlainPenStyle;
        }
    }

    var lines = devState.renderSettings.penStyle.renderFunction(devState.penBuffer);

    if (devState.renderSettings.type === "image/svg+xml" || devState.renderSettings.type === "image/svg") {
        renderVector(lines, devState);
    } else {
        renderRaster(lines, devState);
    }
}
function renderVector(lines, devState) {
    "use strict";
    var scale = devState.renderSettings.scale;
    var width = devState.deviceInfo.displayInfo.width;
    var height = devState.deviceInfo.displayInfo.height;
    var i;

    if (devState.renderSettings.crop) {
        var size = cropLines(lines);
        width = size.width * scale;
        height = size.height * scale;
    } else {
        width *= scale;
        height *= scale;
    }

    var dom = '<svg xmlns="http://www.w3.org/2000/svg" xmlns:st="http://www.scriptel.com/xml/svgsignature" version="1.1" width="' + width + '" height="' + height + '">\n';

    for (i = 0; i < lines.length; i++) {
        var l = lines[i];
        var style = "stroke-linecap: round;";
        if (l.options.lineWidth) {
            style += "stroke-width: " + (l.options.lineWidth * scale) + ";";
        }
        if (l.options.strokeStyle) {
            style += "stroke: " + l.options.strokeStyle + ";";
        }

        var t1 = new Date(l.t1).toISOString();
        var t2 = new Date(l.t2).toISOString();
        dom += '\t<line x1="' + (l.x1 * scale) + '" y1="' + (l.y1 * scale) + '" x2="' + (l.x2 * scale) + '" y2="' + (l.y2 * scale) + '" st:t1="' + t1 + '" st:t2="' + t2 + '" style="' + style + '" />\n';
    }

    dom += "</svg>\n";

    var buf = new Buffer(dom, "utf8");
    var img = new dto.RenderedImage();
    img.type = devState.renderSettings.type;
    img.width = width;
    img.height = height;
    img.data = "data:" + devState.renderSettings.type + ";base64," + buf.toString("base64");
    devState.connection.send(JSON.stringify(img));
}
/**
 * This function attempts to render a raster image (image/png, image/jpeg, image/gif, etc).
 */
function renderRaster(lines, devState) {
    "use strict";
    var scale = devState.renderSettings.scale;
    var width = devState.deviceInfo.displayInfo.width * scale;
    var height = devState.deviceInfo.displayInfo.height * scale;
    var canvas, i, prop;

    if (devState.renderSettings.crop) {
        var size = cropLines(lines);
        canvas = new Canvas(size.width * scale, size.height * scale);
    } else {
        canvas = new Canvas(width, height);
    }

    var ctx = canvas.getContext("2d");
    ctx.antialias = "subpixel";
    ctx.lineCap = "round";
    for (i = 0; i < lines.length; i++) {
        var l = lines[i];
        ctx.beginPath();

        //Copy any user-defined canvas properties into the canvas context.
        for (prop in l.options) {
            if (l.options.hasOwnProperty(prop)) {
                ctx[prop] = l.options[prop];
            }
        }

        //Line Width is special, needs to be scaled.
        if (l.options.lineWidth) {
            ctx.lineWidth = l.options.lineWidth * scale;
        }

        ctx.moveTo(l.x1 * scale, l.y1 * scale);
        ctx.lineTo(l.x2 * scale, l.y2 * scale);

        ctx.stroke();
    }

    var img = new dto.RenderedImage();
    img.type = devState.renderSettings.type;
    img.width = canvas.width;
    img.height = canvas.height;
    canvas.toDataURL(img.type, function (err, str) {
		if (!err) {
			img.data = str;
			devState.connection.send(JSON.stringify(img));
		} else {
			Logger.warning("Problem rendering signature: {0}", [err]);
		}
    });
}
/**
 * This function attempts to intellegently crop a series of line segements such
 * that it can be rendered using the smallest space.
 * @param {Array<module:penstyle~LineSegement>} Line segments to bound and crop.
 * @returns {Object} Returns an object width the bounded width and height.
 */
function cropLines(lines) {
    "use strict";
    var maxX = 0;
    var minX = 1000000;
    var maxY = 0;
    var minY = 1000000;

    var maxWidth = 0;
    var i;

    //Calculate bounding box.
    for (i = 0; i < lines.length; i++) {
        var l = lines[i];
        if (l.x1 < minX) { minX = l.x1; }
        if (l.x2 < minX) { minX = l.x2; }
        if (l.y1 < minY) { minY = l.y1; }
        if (l.y2 < minY) { minY = l.y2; }

        if (l.x1 > maxX) { maxX = l.x1; }
        if (l.x2 > maxX) { maxX = l.x2; }
        if (l.y1 > maxY) { maxY = l.y1; }
        if (l.y2 > maxY) { maxY = l.y2; }

        if (parseInt(l.options.lineWidth, 10) > maxWidth) { maxWidth = parseInt(l.options.lineWidth, 10); }
    }

    //Now need need to subtract all points by minX,minY while remembering to
    //add padding for maxWidth.
    for (i = 0; i < lines.length; i++) {
        lines[i].x1 -= minX;
        lines[i].x2 -= minX;
        lines[i].y1 -= minY;
        lines[i].y2 -= minY;

        lines[i].x1 += maxWidth;
        lines[i].x2 += maxWidth;
        lines[i].y1 += maxWidth;
        lines[i].y2 += maxWidth;
    }

    var width = ((maxX - minX) + (maxWidth * 2));
    var height = ((maxY - minY) + (maxWidth * 2));

    if (width < 1) {
        width = 1;
    }
    if (height < 1) {
        height = 1;
    }
    //Now return the adjusted width/height
    return {"width": width, "height": height};
}

function DeviceWrapper() {
    this.protocolDriver = null;
    this.devHandle = null;
    this.path = "";
    this.uuid = null;
    this.product = "";
    this.manufacturer = "";
    this.vendorId = -1;
    this.productId = -1;
}

/**
 * This function requests that the native HID driver re-enumerate devices.
 * @return {Array<module:dto~DeviceEndpoint>} List of attached devices.
 */
function reloadDevices() {
    "use strict";
    var devList = DeviceManager.getAttachedDevices();
    var devs = [];
	var x, y, device;
    //We need to assign protocol drivers to all devices we can.
    for (x = devList.length - 1; x >= 0; x--) {
        var wrapper = new DeviceWrapper();
        wrapper.path = devList[x].path;
        wrapper.product = devList[x].model;
        wrapper.manufacturer = devList[x].manufacturer;
        wrapper.vendorId = devList[x].vendorId.toString(16);
        wrapper.productId = devList[x].productId.toString(16);
        wrapper.devHandle = devList[x];
        wrapper.protocolDriver = new ScripTouchDevice(devList[x]);
        
        while(wrapper.vendorId.length < 4) {
            wrapper.vendorId = "0"+wrapper.vendorId;
        }

        while(wrapper.productId.length < 4) {
            wrapper.productId = "0"+wrapper.productId;
        }

        var inBlacklist = false;
        for(y = 0; !inBlacklist && y < blackList.length; y++) {
            var bl = blackList[y];
            if(devList[x].vendorId == bl.vendorId && devList[x].productId == bl.productId) {
                Logger.warning("Excluding device {0} because it's in the blacklist.", [wrapper.product]);
                inBlacklist = true;
            }
        }

        if(!inBlacklist) {
            devs[devs.length] = wrapper;
        }
    }

    //Now to look up UUID's for the devices we have.
    for (x = 0; x < devs.length; x++) {
        //Now we need to make sure it's not leased, if it is, use the cached
        //uuid.
        var path = devs[x].path;
        if (leases && leases[path]) {
            devs[x].uuid = leases[path].deviceInfo.uuid;
        } else {
            //We need to open this device quickly and "peek" at the uuid.
            device = devs[x].devHandle;
            try {
                device.open();
                devs[x].uuid = device.getUuidString();
                device.close();
            } catch (e) {
                Logger.warning("Problem getting device UUID: {0}", [e]);
            }
        }
    }

    //Send out a broadcast to the multicast address announcing our new device
    //list.
    if (discovery) {
        discovery.broadcastInstanceInformation(devs);
    }
    return devs;
}

/**
 * Constructor, creates a new instance of this type.
 * @classdesc This type represents a device lease to a WebSocket connection.
 * @constructor
 */
function DeviceLease() {
    "use strict";
    /**
     * The connection associated with this device lease.
     * @public
     * @instance
     * @type {WebSocketConnection}
     */
    this.connection = false;
    /**
     * The device information associated with this lease.
     * @public
     * @instance
     * @type {module:dto~DeviceEndpoint}
     */
    this.deviceInfo = false;
    /**
     * The handle to the actual device.
     * @public
     * @instance
     * @type {module:dto~ScriptelDevice}
     */
    this.deviceHandle = false;
    /**
     * A function handle to the input loop for this device.
     * @public
     * @instance
     * @type {function}
     */
    this.handleInput = false;
    /**
     * A map of region presets defined by the client.
     * @public
     * @instance
     * @type {object<string:array<dto~Region>>}
     */
    this.regionPresets = {};
    this._class = "DeviceLease";
}
module.exports.DeviceLease = DeviceLease;

/**
 * Constructor, creates a new instance of this type.
 * @classdesc This type represents a device's current state.
 * @constructor
 */
function DeviceState() {
    "use strict";
    /**
     * Is the pen presently pressed against the glass?
     * @public
     * @instance
     * @type {boolean}
     */
    this.penDown = false;
    /**
     * Is a button currently depressed?
     * @public
     * @instance
     * @type {boolean}
     */
    this.buttonDown = false;
    /**
     * The region id of the last button our pen was on.
     * @public
     * @instance
     * @type {number}
     */
    this.lastButton = -1;
    /**
     * An array of coordinate events buffered such that we can produce a
     * rendered signature.
     * @public
     * @instance
     * @type {Array<module:ST1500Device~Coordinate>}
     */
    this.penBuffer = [];
    /**
     * The device associated with this state information.
     * @public
     * @instance
     * @type {module:dto~DeviceEndpoint}
     */
    this.deviceInfo = false;
    /**
     * This contains the render settings for this device.
     * @public
     * @instance
     * @type {module:dto~RenderSettings}
     */
    this.renderSettings = false;
    this._class = "DeviceState";
}
module.exports.DeviceState = DeviceState;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This class is sent when the operating system notices a change in
 * USB device membership.
 * @constructor
 * @param {string} Path of the device that had the change.
 * @param {string} Action either 'add' or 'remove'.
 */
function DeviceEndpointChange(path, action) {
    "use strict";
    /**
     * Path of the device that registered the change.
     * @public
     * @instance
     * @type {string}
     */
    this.path = path;
    /**
     * Type of change, either "add" or "remove".
     * @public
     * @instance
     * @type {string}
     */
    this.action = action;
    this._class = "DeviceEndpointChange";
}
module.exports.DeviceEndpointChange = DeviceEndpointChange;


function printVersion() {
	"use strict";
    var versions, lib;
    versions = "\n-\nOmniScript Version\n";
	versions += "  Version Number: " + osVersion.versionNumber + "\n";
	versions += "      Build Date: " + osVersion.buildDate + "\n";
	versions += "    Build Number: " + osVersion.buildNumber + "\n";
	versions += "        Revision: " + osVersion.revision + "\n";
	versions += "        Job Name: " + osVersion.jobName + "\n\n";

	//List Library Versions:
	versions += "Native Library Versions \n";
	for (lib in process.versions) {
        if (process.versions.hasOwnProperty(lib)) {
            var version = process.versions[lib];
            versions += "   " + lib + "::" + version + "\n";
        }
	}
	versions = versions.trim();

	Logger.info(versions);
}

/**
 * This function parses a PNG data url into a node buffer.
 */
function parseDataUrl(str) {
    var match = str.match(/^data:image\/(.+);base64,(.*)$/);
    if(!match) {
        throw "String doesn't appear to be an image data url.";
    } else if(match[1]!=="png") {
        throw "Image format doesn't appear to be PNG.";
    }
    
    return new Buffer(match[2],"base64");
}

function sendMessage(message, connection) {
    var state = getConnectionState(connection);
    var lease = getConnectionLease(connection);
    //We'll give our plugins first whack at this.
    if(!plugins.handleOutboundMessage(message, connection, state, lease, lease.handleInput)) {
        //None of the plugins handled this message, we'll handle it.
        connection.sendUTF(JSON.stringify(message));
    }
}
