/**
 * This module contains processes for turning pen events into signatures.
 * @author Mike Powers <mpowers@scriptel.com>
 * @module penstyle
 */

 /**
 * Constructor, makes a new instance of this type.
 * @classdesc This class represents a line segment.
 * @constructor
 */
function LineSegment(x1, y1, x2, y2, t1, t2, options) {
    "use strict";
    /**
     * The horizontal component of the first point.
     * @public
     * @instance
     * @type {number}
     */
    this.x1 = x1 || 0;
    /**
     * The vertical component of the first point.
     * @public
     * @instance
     * @type {number}
     */
    this.y1 = y1 || 0;
    /**
     * The horizontal component of the second point.
     * @public
     * @instance
     * @type {number}
     */
    this.x2 = x2 || 0;
    /**
     * The vertical component of the second point.
     * @public
     * @instance
     * @type {number}
     */
    this.y2 = y2 || 0;

    /**
     * The time at which the first point was observed.
     * @public
     * @instance
     * @type {number}
     */
    this.t1 = t1;

     /**
     * The time at which the first second was observed.
     * @public
     * @instance
     * @type {number}
     */
    this.t2 = t2;

    /**
     * Options that are sent to the renderer for this line segment. Options
     * sent along in the rendering settings override the defaults.
     * @public
     * @instance
     * @type {Object<string,string>}
     */
    this.options = {
        "lineWidth": 1,
        "fillStyle": "#000000",
        "strokeStyle": "#000000"
    };

    //Copy any options passed in, overwrite if nessisary.
    if (options) {
        var opt;
        for (opt in options) {
            if (options.hasOwnProperty(opt)) {
                this.options[opt] = options[opt];
            }
        }
    }
}
module.exports.LineSegment = LineSegment;

/**
 * This pen style algorithm uses the angle of points in order to determine
 * relative width.
 */
function ChiselPenStyle(events) {
    "use strict";
    var i;
    if (events.length < 2) {
        return [];
    }
    var lines = [];

    var minWidth = 0.1;
    var maxWidth = 1.75;

    for (i = 1; i < events.length; i++) {
        var p1 = events[i - 1];
        var p2 = events[i];
        if (p1._class !== "PenUp") {
            var angle = Math.atan2(p1.x - p2.x, p1.y - p2.y);

            if (angle < 0) {
                angle += Math.PI;
            }

            if (angle <= (Math.PI / 2)) {
                angle = Math.abs(angle - (Math.PI / 4));
            } else {
                angle = Math.abs(angle - (Math.PI / 2) - (Math.PI / 4));
                angle = (Math.PI / 2) - angle;
            }

            var pct = 1 - (angle / (Math.PI / 2));

            var line = new LineSegment(p1.x, p1.y, p2.x, p2.y, p1.time, p2.time, this.options);
            line.options.lineWidth = ((maxWidth - minWidth) * pct) + minWidth;
            lines[lines.length] = line;
        }
    }
    return lines;
}
module.exports.ChiselPenStyle = ChiselPenStyle;

/**
 * This pen style simply plays "connect-the-dots" with a pre-determined width.
 */
function PlainPenStyle(events) {
    "use strict";
    var i;
    if (events.length < 2) {
        return [];
    }

    var lines = [];
    for (i = 1; i < events.length; i++) {
        if (events[i - 1]._class !== "PenUp") {
            lines[lines.length] = new LineSegment(events[i - 1].x, events[i - 1].y, events[i].x, events[i].y, events[i - 1].time, events[i].time, this.options);
        }
    }
    return lines;
}
module.exports.PlainPenStyle = PlainPenStyle;

/**
 * This pen style uses relative distances to determine the width of lines
 * creates a "bleeding" effect similar to that of inkwell pens.
 */
function InkwellPenStyle(events) {
    "use strict";
    var i, j;
    if (events.length < 2) {
        return [];
    }

    var lines = [];

    var minWidth = 0.5;
    var maxWidth = 1.5;

    var distHist = [0, 0, 0, 0, 0];

    for (i = 1; i < events.length; i++) {
        var p1 = events[i - 1];
        var p2 = events[i];
        if (p1._class !== "PenUp") {
            var dist = Math.sqrt(Math.pow(p2.x - p1.x, 2) + Math.pow(p2.y - p1.y, 2));
            distHist.splice(0, 1); //Drop the first element.
            distHist[distHist.length] = dist; //Add our new element.

            //Get average velocity for the past few samples.
            var velocity = 0;
            for (j = 0; j < distHist.length; j++) {
                velocity += distHist[j];
            }
            velocity /= distHist.length;

            var pct = (velocity / 3 <= 1) ? 1 - (velocity / 3) : 0;

            var line = new LineSegment(p1.x, p1.y, p2.x, p2.y, p1.time, p2.time, this.options);
            line.options.lineWidth = ((maxWidth - minWidth) * pct) + minWidth;
            lines[lines.length] = line;
        }
    }
    return lines;
}
module.exports.InkwellPenStyle = InkwellPenStyle;
