/**
 * This module contains all of the logic required to talk to a ScripTouch ST1500
 * at the USB HID Layer
 * @author Scriptel Engineering <EngineeringDepartment@scriptel.com>
 * @module ScripTouchDevice
 */
var crypto = require("crypto");
var dto = require("../lib/scriptel-dto.js");

function ScripTouchDevice() {
    "use strict";
    this.features = ["screen", "pen"];
}
module.exports = ScripTouchDevice;

ScripTouchDevice.prototype.getDeviceInfo = function (dev) {
    "use strict";
    var devInfo = new dto.ScriptelDevice();
	var retr, x, i, y;

    devInfo.model = dev.model;
    devInfo.manufacturer = dev.manufacturer;
    devInfo.vendorId = dev.vendorId;
    devInfo.productId = dev.productId;

    try {
        retr = dev.getVersionInfo();
        devInfo.softwareVersion = new dto.SoftwareVersion();
        devInfo.softwareVersion.kernelVersion.major = retr.kernelMajor;
        devInfo.softwareVersion.kernelVersion.minor = retr.kernelMinor;
        devInfo.softwareVersion.kernelVersion.release = retr.kernelRelease;
        devInfo.softwareVersion.firmwareVersion.major = retr.firmwareMajor;
        devInfo.softwareVersion.firmwareVersion.minor = retr.firmwareMinor;
        devInfo.softwareVersion.firmwareVersion.release = retr.firmwareRelease;
        devInfo.softwareVersion.asicSignature = retr.asicSignature;
    } catch (e1) {
        throw "Problem getting software version from device: " + e1.message;
    }

    try {
        devInfo.coordinateRange = dev.getCoordinateRange();
    } catch(ee1) {
        //No dice, must not be supported.
        devInfo.coordinateRange = {
            "xMin": 0,
            "xMax": 0xFFFF,
            "yMin": 0,
            "yMax": 0xFFFF
        };
    }

    try {
        devInfo.capabilities = dev.getCapabilities();
    } catch(ee2) {
        //Not supported, that's ok.
    }

    //Test to see if the firmware supports serial numbers.
    if(devInfo.softwareVersion.firmwareVersion.major >= 3 && devInfo.softwareVersion.firmwareVersion.major >= 1 && devInfo.softwareVersion.firmwareVersion.release >= 24) {
        try {
            devInfo.serialNumber = dev.getSerialNumber();
        } catch(er1) {
            throw "Problem getting serial number from the device: " + er1.message;
        }
    }

    if (this.features.indexOf("screen") >= 0) {
        try {
            retr = dev.getDisplaySettings();
            devInfo.displaySettings = new dto.DisplaySettings();
            devInfo.displaySettings.brightness = retr.brightness;
            devInfo.displaySettings.contrast = retr.contrast;
            devInfo.displaySettings.red = retr.red;
            devInfo.displaySettings.green = retr.green;
            devInfo.displaySettings.blue = retr.blue;
            devInfo.displaySettings.command = retr.setCommand;
        } catch (e2) {
            devInfo.displaySettings = false;
        }
    }

    try {
        retr = dev.getDisplayInfo();
        devInfo.displayInfo = new dto.DisplayInfo();
        devInfo.displayInfo.width = retr.width;
        devInfo.displayInfo.height = retr.height;
        devInfo.displayInfo.colorDepth = retr.colorDepth;
        devInfo.displayInfo.characterWidth = retr.textWidth;
        devInfo.displayInfo.characterHeight = retr.textHeight;
        devInfo.displayInfo.regionCount = retr.regionCount;
        devInfo.displayInfo.captionLength = retr.maxCaptionLength;

        try {
            retr = dev.getRegions();
            devInfo.regions = this.readRegions(devInfo, retr);
        } catch (e3) {
            throw "Problem getting display region information from the device: " + e3.message;
        }
    } catch (e4) {
        throw "Problem getting display information from device: " + e4.message;
    }


    try {
        retr = dev.getOperatingModes();
        devInfo.operatingMode = new dto.OperatingMode();
        devInfo.operatingMode.supportedModes = new dto.SupportedOperatingModes();
        devInfo.operatingMode.supportedModes.touch = (retr.supportedModes.indexOf("TOUCH") >= 0);
        devInfo.operatingMode.supportedModes.pen = (retr.supportedModes.indexOf("PEN") >= 0);
        devInfo.operatingMode.supportedModes.autoTouch = (retr.supportedModes.indexOf("AUTOTOUCH") >= 0);
        devInfo.operatingMode.supportedModes.securePen = false;
        devInfo.operatingMode.supportedModes.secureAutoTouch = false;
        devInfo.operatingMode.supportedModes.command = (retr.supportedModes.indexOf("COMMAND") >= 0);

        devInfo.operatingMode.acquisitionMode = retr.acquisitionMode;
        devInfo.operatingMode.touchMode = retr.touchMode;
        devInfo.operatingMode.penMode = retr.penMode;
    } catch (e5) {
        throw "Problem getting operating mode from device: " + e5.message;
    }

    try {
        var errTable = [];
        for (x = 0; x < 2049; x++) {
            errTable[x] = 0;
        }
        errTable[0] = 26;
        errTable = dev.getErrorCorrectionTable();

        var errCor = new dto.ScreenErrorCorrection();
        if (errTable[0] !== 0xFE) {
            errCor.valid = false;
        }

        errCor.xMin = (errTable[1] << 8) + errTable[2];
        errCor.xDelta = (errTable[3] << 8) + errTable[4];
        errCor.yMin = (errTable[5] << 8) + errTable[6];
        errCor.yDelta = (errTable[7] << 8) + errTable[8];
        errCor.xOffset = errTable[17];
        errCor.yOffset = errTable[18];

        var checksum = 0;
        for (x = 0; x < errTable.length - 1; x++) {
            checksum += errTable[x];
        }
        checksum = (0xA5 - checksum) & 0xFF;

        //Check the checksum
        if (checksum !== errTable[errTable.length - 1]) {
            errCor.valid = false;
        }

        //Unpack the table.
        var idx = 0;

        for (i = 19; i < errTable.length - 1; i += 3) {
            x = (errTable[i] << 4) + (errTable[i + 1] & 0xF0 >> 4);
            y = (errTable[i + 2] << 4) + (errTable[i + 1] & 0xF);

            if (x < errCor.boundX1) {
                errCor.boundX1 = x;
            }
            if (x > errCor.boundX2) {
                errCor.boundX2 = x;
            }

            if (y < errCor.boundY1) {
                errCor.boundY1 = y;
            }
            if (y > errCor.boundY2) {
                errCor.boundY2 = y;
            }

            errCor.correctionMap[0][idx] = x;
            errCor.correctionMap[1][idx] = y;
            idx++;
        }

        devInfo.errorCorrection = errCor;
    } catch (e6) {
        throw "Problem getting error correction information: " + e6.message;
    }

    try {
        devInfo.calibrationMode = this.getCalibrationData(dev);
    } catch (e7) {
        throw "Problem getting calibration mode info from device: " + e7.message;
    }

    if (!devInfo.calibrationMode || !devInfo.calibrationMode.calibrating) {
        this.clearScreen(dev, devInfo);
    }

    return devInfo;
};
ScripTouchDevice.prototype.sendSprite = function (dev, devInfo, sprite) {
    "use strict";
    if (sprite.width < 1 || sprite.height < 1 || sprite.x < 0 || sprite.y < 0) {
        throw "Invalid dimensions or position.";
    } else if (sprite.graphic.length > 2039) {
        throw "Raster is too large to send.";
    } else if (sprite.width > devInfo.displayInfo.width) {
        throw "Image is wider than display: "+sprite.width+" > "+devInfo.displayInfo.width;
    } else if (sprite.height > devInfo.displayInfo.height) {
        throw "Image is taller than display: "+sprite.height+" > "+devInfo.displayInfo.height;
    } else if(sprite.x+sprite.width > devInfo.displayInfo.width) {
        throw "Image would draw off the right of the screen.";
    } else if(sprite.y+sprite.height > devInfo.displayInfo.height) {
        throw "Image would draw off the bottom of the screen.";
    }
    dev.writeDisplayChunk(sprite);
};
ScripTouchDevice.prototype.convertBitmapToSpriteBW = function (buffer) {
    "use strict";
    if(buffer.getContext) {
        var canvas = buffer;
        var ctx = canvas.getContext("2d");
    } else {
        var Canvas = require("canvas");
        var Image = Canvas.Image;

        var img = new Image();
        img.src = buffer;

        var canvas = new Canvas(img.width, img.height);
        var ctx = canvas.getContext("2d");
        ctx.drawImage(img, 0, 0, img.width, img.height);
    }

    var i = 0;
    var retr = new Array(2039);
    for(i = 0; i < retr.length; i++) {
        retr[i] = 0;
    }
    var bytePos = 0;
    var bitPos = 0;
    var x, y;
    for (x = 0; x < canvas.width; x++) {
        for (y = 0; y < canvas.height; y++) {
            var color = ctx.getImageData(x, y, 1, 1).data;
            var black = ((color[0] + color[1] + color[2]) / 3) < 0x80;

            if (!retr[bytePos]) {
                retr[bytePos] = 0;
            }

            if (black) {
                retr[bytePos] |= 1 << (7 - bitPos);
            }

            if ((bitPos + 1) % 8 === 0) {
                bitPos = 0;
                bytePos++;
            } else {
                bitPos++;
            }
        }
    }
    var sprite = new Sprite();
    sprite.x = 0;
    sprite.y = 0;
    sprite.width = canvas.width;
    sprite.height = canvas.height;
    sprite.bitsPerPixel = 1;
    sprite.graphic = retr;
    
    //We don't need to worry about splitting these, since none of the screens
    //are big enough to exhaust the display size.
    return [sprite];
};
ScripTouchDevice.prototype.convertBitmapToSpriteGray = function (buffer) {
    "use strict";
    if(buffer.getContext) {
        var canvas = buffer;
        var ctx = canvas.getContext("2d");
    } else {
        var Canvas = require("canvas");
        var Image = Canvas.Image;

        var img = new Image();
        img.src = buffer;

        var canvas = new Canvas(img.width, img.height);
        var ctx = canvas.getContext("2d");
        ctx.drawImage(img, 0, 0, img.width, img.height);
    }
    var chunks = Math.ceil((canvas.width * canvas.height) / ((2049-10) * 2));
    var sprites = [];
    
    for (var c=0;c<chunks;c++) {
        var i = 0;
        var retr = new Array(2039);
        for(i = 0; i < retr.length; i++) {
            retr[i] = 0;
        }
        var bytePos = 0;
        var bitPos = 0;
        var x, y;
        var w = Math.floor(canvas.width / chunks);
        
        var sX = w * c;
        var eX = (c + 1 !== chunks)?w * (c + 1):sX + (canvas.width - sX);
        
        for (x = sX; x < eX; x++) {
            for (y = 0; y < canvas.height; y++) {
                var color = ctx.getImageData(x, y, 1, 1).data;
                var gray = Math.round((((color[0] + color[1] + color[2]) / 3) / 255) * 0xF);

                if (!retr[bytePos]) {
                    retr[bytePos] = 0;
                }

                retr[bytePos] |= gray << bitPos;

                if (bitPos > 0 && bitPos % 4 === 0) {
                    bitPos = 0;
                    bytePos++;
                } else {
                    bitPos += 4;
                }
            }
        }
        
        var sprite = new Sprite();
        sprite.x = sX;
        sprite.y = 0;
        sprite.width = eX-sX;
        sprite.height = canvas.height;
        sprite.bitsPerPixel = 4;
        sprite.graphic = retr;
        sprites[sprites.length] = sprite;
    }
    return sprites;
};
ScripTouchDevice.prototype.getCalibrationData = function (dev) {
    "use strict";
    var retr = dev.getCalibrationState();
    var mode = new dto.CalibrationMode();
    mode.calibrating = retr.calibrationMode;
    var point =  retr.point;
    var pointName = "Unknown";
    if (point === "CENTER") {
        pointName = "Center";
    } else if (point === "UPPERLEFT") {
        pointName = "Upper Left";
    } else if (point === "LOWERLEFT") {
        pointName = "Lower Left";
    } else if (point === "LOWERRIGHT") {
        pointName = "Lower Right";
    } else if (point === "UPPERRIGHT") {
        pointName = "Upper Right";
    }
    mode.calibrationPoint = pointName;
    return mode;
};
ScripTouchDevice.prototype.forceRecalibration = function (dev) {
    "use strict";
    dev.calibrate();
};
ScripTouchDevice.prototype.resetScreen = function (dev, devInfo) {
    var regions = dev.getDefaultRegions();
    var i;
    for(i = 0; i < regions.length; i++) {
        if(regions[i].type === "UNKNOWN") {
            regions[i].type = "NULL";
        }
        if(regions[i].inkEnable) {
            regions[i].inkEnable = true;
        } else {
            regions[i].inkEnable = false;
        }

        dev.setRegion(regions[i]);
    }
    
    return this.readRegions(devInfo, dev.getRegions());
};
ScripTouchDevice.prototype.clearScreen = function (dev, devInfo) {
    "use strict";
    dev.clearScreen();
};
ScripTouchDevice.prototype.updateRegion = function (dev, devInfo, region) {
    "use strict";
    var r = this.convertRegionToLibrary(region);
    dev.setRegion(r);
};
ScripTouchDevice.prototype.convertRegionFromLibrary = function(region) {
    "use strict";
    var r = new dto.Region();
    r.id = region.regionNumber;
    r.x1 = region.x1;
    r.x2 = region.x2;
    r.y1 = region.y1;
    r.y2 = region.y2;
    r.foregroundColor = [region.colorForeground.red, region.colorForeground.green, region.colorForeground.blue, region.colorForeground.alpha];
    r.backgroundColor = [region.colorBackground.red, region.colorBackground.green, region.colorBackground.blue, region.colorBackground.alpha];
    r.flags = new dto.RegionFlags();
    r.flags.acceptsTouch = region.acceptTouch;
    r.flags.decodeRegion = region.decode;
    r.flags.visible = region.visible;
    r.flags.inUse = region.inUse;
    r.flags.hasFrame = region.hasFrame;

    if(region.type === "CONTAINER") {
        r.containerType = new dto.RegionContainer();
        r.containerType.flags = new dto.RegionContainerFlags();
        r.containerType.flags.inkEnabled = region.inkEnable;
    } else if(region.type === "BUTTON") {
        r.buttonType = new dto.RegionButton();
        r.buttonType.parentContainer = region.parent;
        r.buttonType.flags = new dto.RegionButtonFlags();
        r.buttonType.flags.enabled = region.enable;
        r.buttonType.flags.highlightOnSelect = region.highlightOnSelect;
        r.buttonType.flags.clearOnSelect = region.clearOnSelect;
        r.buttonType.flags.confirmOnSelect = false; //huh.
        r.buttonType.caption = region.caption
    } else if(region.type === "LINE") {
        r.lineType = new dto.RegionLine();
        r.lineType.parentContainer = region.parent;
        r.lineType.lineWidth = region.width;
    }
    return r;
};
ScripTouchDevice.prototype.convertRegionToLibrary = function(r) {
    "use strict";

    var region = {};
    region.regionNumber = r.id;
    region.x1 = r.x1;
    region.x2 = r.x2;
    region.y1 = r.y1;
    region.y2 = r.y2;
    if(region.colorForeground && region.colorForeground.length == 4) {
        region.colorForeground = {"red": r.foregroundColor[0], "green": r.foregroundColor[1], "blue": r.foregroundColor[2], "alpha": r.foregroundColor[3]};
    } else {
        region.colorForeground = {"red": 0x00, "green": 0x00, "blue": 0x00, "alpha": 0xFF};
    }
    if(region.colorBackground && region.colorBackground.length == 4) {
        region.colorBackground = {"red": r.backgroundColor[0], "green": r.backgroundColor[1], "blue": r.backgroundColor[2], "alpha": r.backgroundColor[3]};
    } else {
        region.colorBackground = {"red": 0xFF, "green": 0xFF, "blue": 0xFF, "alpha": 0xFF};
    }

    if(r.flags) {
        region.acceptTouch = (r.flags.acceptsTouch) ? true : false;
        region.decode = (r.flags.decodeRegion) ? true : false;
        region.visible = (r.flags.visible) ? true : false;
        region.inUse = (r.flags.inUse) ? true : false;
        region.hasFrame = (r.flags.hasFrame) ? true : false;
    } else {
        region.acceptTouch = true;
        region.decode = true;
        region.visible = true;
        region.inUse = true;
        region.hasFrame = false;
    }

    //Set defaults.
    region.inkEnable = false;
    region.parent = 0;
    region.enable = false;
    region.highlightOnSelect = false;
    region.clearOnSelect = false;
    region.caption = "";
    region.width = 0;
    region.reserved1 = 0;
    region.reserved2 = 0;
    region.containerReserved1 = 0;
    region.containerReserved2 = 0;
    region.containerReserved3 = 0;
    region.lineReserved = 0;
    region.buttonReserved = 0;

    if(r.containerType) {
        region.type = "CONTAINER";
        if(r.containerType.flags) {
            region.inkEnable = (r.containerType.flags.inkEnabled) ? true : false;
        } else {
            region.inkEnable = true;
        }
    } else if(r.buttonType) {
        region.type = "BUTTON";
        region.parent = r.buttonType.parentContainer;
        if(r.buttonType.flags) {
            region.enable = (r.buttonType.flags.enabled) ? true : false;
            region.highlightOnSelect = (r.buttonType.flags.highlightOnSelect) ? true : false;
            region.clearOnSelect = (r.buttonType.flags.clearOnSelect) ? true : false;
        } else {
            region.enable = true;
            region.highlightOnSelect = true;
            region.clearOnSelect = true;
        }
        region.caption = r.buttonType.caption;
    } else if(r.lineType) {
        region.type = "LINE";
        region.parent = r.lineType.parentContainer;
        region.width = r.lineType.lineWidth;
    } else {
        region.type = "NULL";
    }
    return region;
};

ScripTouchDevice.prototype.readRegions = function (devInfo, arr) {
    "use strict";
    var i;
    var retr = [];
    for(i = 0; i < arr.length; i++) {
        retr[i] = this.convertRegionFromLibrary(arr[i]);
    }
    return retr;
};
ScripTouchDevice.prototype.strFromBytes = function (arr, start, len) {
    "use strict";
    var str = "";
    var x;
    for (x = start; x < start + len; x++) {
        if (arr[x] !== 0) {
            str += String.fromCharCode(arr[x]);
        }
    }
    return str;
};
ScripTouchDevice.prototype.readCallback = function (data, devInfo) {
    "use strict";
    if(data.type === "ExtendedCoordinate") {
        var coord = new Coordinate();
        coord.x = (data.x - devInfo.coordinateRange.xMin) / (devInfo.coordinateRange.xMax - devInfo.coordinateRange.xMin);
        coord.y = (data.y - devInfo.coordinateRange.yMin) / (devInfo.coordinateRange.yMax - devInfo.coordinateRange.yMin);
        coord.region = data.region;
        coord.flags = new CoordinateFlags();
        coord.flags.penSwitchOn = data.sampleInfo.down;
        coord.flags.coordinateValid = true;
        return coord;
    } else if(data.type === "Coordinate") {
        var coord = new Coordinate();
        coord.x = (data.x - devInfo.coordinateRange.xMin) / (devInfo.coordinateRange.xMax - devInfo.coordinateRange.xMin);
        coord.y = (data.y - devInfo.coordinateRange.yMin) / (devInfo.coordinateRange.yMax - devInfo.coordinateRange.yMin);
        coord.flags = new CoordinateFlags();
        coord.flags.penSwitchOn = data.sampleInfo.down;
        coord.flags.coordinateValid = true;
        return coord;
    } else if(data.type === "ButtonDown") {
        var evt = new dto.ButtonDown();
        evt.id = data.region.regionNumber;
        evt.x = (data.x - devInfo.coordinateRange.xMin) / (devInfo.coordinateRange.xMax - devInfo.coordinateRange.xMin);
        evt.y = (data.y - devInfo.coordinateRange.yMin) / (devInfo.coordinateRange.yMax - devInfo.coordinateRange.yMin);
        evt.label = data.region.caption;
        return evt;
    } else if(data.type === "ButtonPress") {
        var evt = new dto.ButtonPress();
        evt.id = data.region.regionNumber;
        evt.x = (data.x - devInfo.coordinateRange.xMin) / (devInfo.coordinateRange.xMax - devInfo.coordinateRange.xMin);
        evt.y = (data.y - devInfo.coordinateRange.yMin) / (devInfo.coordinateRange.yMax - devInfo.coordinateRange.yMin);
        evt.label = data.region.caption;
        evt.clear = data.region.clearOnSelect;
        evt.accept = data.region.caption !== "Cancel";
        return evt;
    } else if(data.type === "ButtonUp") {
        var evt = new dto.ButtonUp();
        evt.id = data.region.regionNumber;
        evt.x = (data.x - devInfo.coordinateRange.xMin) / (devInfo.coordinateRange.xMax - devInfo.coordinateRange.xMin);
        evt.y = (data.y - devInfo.coordinateRange.yMin) / (devInfo.coordinateRange.yMax - devInfo.coordinateRange.yMin);
        evt.label = data.region.caption;
        return evt;
    } else if(data.type === "ButtonMove") {
        //We never handled these traditionally.
    } else if(data.type === "MagCardData") {
        var card = new dto.CardSwipe();

        if (data.format === "Financial") {
            card.format = "financial";
        } else if (data.format === "Identification") {
            card.format = "personalid";
        } else {
            card.format = "unknown";
        }

        card.trackOneError = this.getTrackStatus(data.track1Status);
        card.trackTwoError = this.getTrackStatus(data.track2Status);
        card.trackThreeError = this.getTrackStatus(data.track3Status);

        card.trackOne = data.track1;
        card.trackTwo = data.track2;
        card.trackThree = data.track3;
        return card;
    }
};
ScripTouchDevice.prototype.getTrackStatus = function (b) {
    "use strict";
    var retr;
    if (b === "Valid") {
        retr = false;
    } else if (b === "ParityError") {
        retr = "Parity Error(s)";
    } else if (b === "UnknownFormat") {
        retr = "Unknown Format";
    } else if (b === "Corrupt") {
        retr = "Corrupt Track";
    } else {
        retr = "Unknown Error";
    }
    return retr;
};
ScripTouchDevice.prototype.getDeviceUUID = function (dev, devInfo) {
    "use strict";
    dev.getUuidString();
};
ScripTouchDevice.prototype.setSerialNumber = function(dev, devInfo, serial) {
    "use strict";
    if(devInfo.softwareVersion.firmwareVersion.major >= 3 && devInfo.softwareVersion.firmwareVersion.major >= 1 && devInfo.softwareVersion.firmwareVersion.release >= 24) {
        dev.setSerialNumber(serial);
    } else {
        return new Error("Device firmware doesn't support serial number updates, please update.");
    }
};
ScripTouchDevice.prototype.updateDisplaySettings = function(dev, devInfo, settings) {
    "use strict";
    settings.setCommand = 0;
    dev.setDisplaySettings(settings);
};

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc This type is responsible for storing information regarding a
 * pen-coordinate.
 * @constructor
 */
function Coordinate() {
    "use strict";
    /**
     * Stores the x coordinate as a percentage (0 to 1) of the screen's width.
     * @public
     * @instance
     * @type {number}
     */
    this.x = -1;
    /**
     * Stores the y coordinate as a percentage (0 to 1) of the screen's height.
     * @public
     * @instance
     * @type {number}
     */
    this.y = -1;
    /**
     * Stores the region that this point intersects according to the device.
     * @public
     * @instance
     * @type {number}
     */
    this.region = 0;
    /**
     * Flags associated with this coordinate.
     * @public
     * @instance
     * @type {module:omniscript~CoordinateFlags}
     */
    this.flags = undefined;
    this._class = "Coordinate";
}
module.exports.Coordinate = Coordinate;
/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc Stores flags associated with a coordinate.
 * @constructor
 */
function CoordinateFlags() {
    "use strict";
    /**
     * Is the pen switch depressed (is the pen against the glass)?
     * @public
     * @instance
     * @type {boolean}
     */
    this.penSwitchOn = false;
    /**
     * Is the coordinate valid?
     * @public
     * @instance
     * @type {boolean}
     */
    this.coordinateValid = false;
    this._class = "CoordinateFlags";
}
module.exports.CoordinateFlags = CoordinateFlags;

/**
 * Constructor, creates a new instance of this type.
 * @class
 * @classdesc Stores a sprite that can be sent to the device.
 * @constructor
 */
function Sprite() {
    "use strict";
    /**
     * The horizontal coordinate of the top left corner of the sprite in pixels.
     * @public
     * @instance
     * @type {number}
     */
    this.x = -1;
    /**
     * The vertical coordinate of the top left corner of the sprite in pixels.
     * @public
     * @instance
     * @type {number}
     */
    this.y = -1;
    /**
     * The width of the sprite in pixels.
     * @public
     * @instance
     * @type {number}
     */
    this.width = -1;
    /**
     * The height of the sprite in pixels.
     * @public
     * @instance
     * @type {number}
     */
    this.height = -1;
    /**
     * The color bit depth of the sprite (right now only 1 and 4 are supported).
     * @public
     * @instance
     * @type {number}
     */
    this.bitsPerPixel = 1;
    /**
     * This is a byte array containing the encoded raster.
     * @public
     * @instance
     * @type {array<number>}
     */
    this.graphic = [];
    this._class = "Sprite";
}
