var http              = require("http");
var https             = require("https");
var path              = require("path");
var fs                = require("fs");
var WebSocketServer   = require("websocket").server;
var Logger            = require("../lib/scriptel-logger.js");

/*
 * A simple static web server, used to host our static content.
 */
function ScriptelWebServer(config) {
    "use strict";
    this.config = config;
    Logger.setConfiguration(config);
    this.webRoot = path.resolve(config.webServer.webRoot);
    this.httpServer = false;
    this.httpPort = config.webServer.httpPort;
    this.httpsServer = false;
    this.httpsPort = config.webServer.httpsPort;
    this.wsServer = false;
    this.wssServer = false;
    this.mimeTypes = config.webServer.mimeTypes;

	var listenAddr = config.webServer.listenAddress;
	if (listenAddr === "*") {
		listenAddr = undefined;
	}

    var t = this;
    //If configured, start HTTP
    if (config.webServer.http) {
        this.httpServer = http.createServer(function (request, response) {
            t.handleRequest(request, response);
        });
        this.httpServer.on('error', (err, socket) => {
            //console.log(err);
            console.log("Could not listen on port " + this.httpPort + ". Aborting.")
            process.exit(80);
        });
        this.httpServer.listen(this.httpPort, listenAddr, 511, function () {
            t.httpServerStartup();
        });
    }
    //If Configured, start HTTPS
    if (config.webServer.https) {
        var options = {"key": fs.readFileSync(config.webServer.httpsKey), "cert": fs.readFileSync(config.webServer.httpsCert)};
        this.httpsServer = https.createServer(options, function (request, response) {
            t.handleRequest(request, response);
        });
        this.httpsServer.on('error', (err, socket) => {
            //console.log(err);
            console.log("Could not listen on port " + this.httpsPort + ". Aborting.")
            process.exit(443);
        });
        this.httpsServer.listen(this.httpsPort, listenAddr, 511, function () {
            t.httpsServerStartup();
        });
    }
}
ScriptelWebServer.prototype.httpServerStartup = function () {
    "use strict";
    Logger.info("Scriptel HTTP Server started on port {0}", [this.httpPort]);
};
ScriptelWebServer.prototype.httpsServerStartup = function () {
    "use strict";
    Logger.info("Scriptel HTTPS Server started on port {0}", [this.httpsPort]);
};
ScriptelWebServer.prototype.handleRequest = function (request, response) {
    "use strict";
    try {
        var file = this.webRoot + this.uriParser(request.url);
        try {
            var stat = fs.statSync(file);
        } catch(e) {
            //Not a file, apparently.
        }

        if (!stat || (!stat.isFile() && !stat.isDirectory())) {
            response.writeHead(404);
            response.end("404 Not Found\n");
            return;
        } else if (stat.isDirectory()) {
            file += "/index.html";
            stat = fs.statSync(file);
            if (!stat.isFile()) {
                response.writeHead(404);
                response.end("404 Not Found\n");
                return;
            }
        }

        var mimeType = "application/octet-stream";
        var idx = file.lastIndexOf(".");
        if (idx >= 0) {
            var ext = file.substring(idx).toLowerCase();
            if (this.mimeTypes[ext]) {
                mimeType = this.mimeTypes[ext];
            }
        }

        fs.readFile(file, "binary", function (err, data) {
            if (!err) {
                response.writeHead(200, {"Content-type": mimeType});
                response.write(data, "binary");
                response.end();
            } else {
				Logger.warning("Problem reading file: {0}", [err]);
            }
        });
    } catch (e) {
        response.writeHead(500);
        response.end(JSON.stringify({"_class": "ScriptelException", "message": e}));
    }
};
ScriptelWebServer.prototype.uriParser = function (str) {
    "use strict";
    var x;
    str = decodeURIComponent(str);
    //Remove any forward slashes, don't risk these counting as "escapes".
    str.replace("\\", "");

    var chunks = str.split("/");

    if (chunks[0] !== "") {
        throw "URL contains a protocol.";
    }

    var level = -2;
    for (x = 0; x < chunks.length; x++) {
        level += 1;
        if (chunks[x] === "..") {
            level -= 2;
        }
    }
    if (level < 0) {
        throw "URL tries to escape web root.";
    }

    var newUri = "/";
    for (x = 0; x < chunks.length; x++) {
        if (chunks[x] !== "") {
            newUri += chunks[x];
            if (x + 1 < chunks.length) {
                newUri += "/";
            }
        }
    }
    return path.normalize(newUri);
};
ScriptelWebServer.prototype.registerWebSocketHandler = function (func) {
    "use strict";
    if (this.config.webServer.http) {
        this.wsServer = new WebSocketServer({"httpServer": this.httpServer, "maxReceivedMessageSize": "10MiB", "maxReceivedFrameSize": "10MiB"});
        this.wsServer.on("request", func);
    }

    if (this.config.webServer.https) {
        this.wssServer = new WebSocketServer({"httpServer": this.httpsServer, "maxReceivedMessageSize": "10MiB", "maxReceivedFrameSize": "10MiB"});
        this.wssServer.on("request", func);
    }
};
module.exports = ScriptelWebServer;