/* OmniScript Example - Pong - Copyright 2013 - Scriptel Corporation
 * This demo application shows how to develop an application that takes
 * advantage of Scriptel's OmniScript product to interact with one or two
 * ScripTouch signature pads spanning one or two OmniScript instances.
 */

/**
 * Initializer, instantiates a new instance of Pong.
 */
function init() {
    var pong = new Pong();
    pong.registerHandlers();
    pong.switchScreen("main","menu");
}

/**
 * Constructor, creates a new object of this type.
 * @constructor
 * @classdesc Pong class, contains all game logic.
 */
function Pong() {
    /**
     * Stores current WebSocket connections and some
     * metadata about them.
     */
    this.connections = [];
    /**
     * Our main drawing canvas, displayed to the user
     * and also pushed to the device.
     */
    this.canvas = document.getElementById("pad");
    /**
     * Main drawing context.
     */
    this.ctx = this.canvas.getContext("2d");
    /**
     * Previous frame canvas, used to determine the bounding
     * box around the content we need to send to the pads.
     */
    this.lastFrame = document.getElementById("lastpad");
    /**
     * Previous frame context.
     */
    this.lastCtx = this.lastFrame.getContext("2d");
    /**
     * Difference canvas, used to copy differences between main
     * and previous to this canvas in order to push it to the
     * device.
     */
    this.diffFrame = document.getElementById("diff");
    /**
     * Difference context.
     */
    this.diffCtx = this.diffFrame.getContext("2d");
    /**
     * State for player one's paddle.
     */
    this.p1 = {
        "position": 32,
        "hPos": 0,
        "score": 0,
        "height": 20,
        "width": 5
    };
    /**
     * State for player two's paddle.
     */
    this.p2 = {
        "position": 32,
        "hPos": 234,
        "score": 0,
        "height": 20,
        "width": 5
    };
    /**
     * State for the ball.
     */
    this.ball = {
        "x": 120,
        "y": 32,
        "dx": 3,
        "dy": 3,
        "radius": 3
    };
    /**
     * In two player mode this is used to determine
     * which connection is player one.
     */
    this.connectionOne = undefined;
    /**
     * In two player mode this is used to determine
     * which connection is player two.
     */
    this.connectionTwo = undefined;
    /**
     * Contains the score limit for the game.
     */
    this.scoreLimit = 3;
    /**
     * Number of seconds to count down before the game
     * begins.
     */
    this.countDown = 5;
    /**
     * Whether or not the game is currently being played.
     */
    this.started = false;
    /**
     * Framerate of the game in milliseconds-per-frame.
     */
    this.redraw = 100;
}
/**
 * This method adds event handlers to all of the HTML elements
 * we'll be using as part of the game.
 */
Pong.prototype.registerHandlers = function() {
    var t = this;
    document.getElementById("main_onep_button").onclick = function() {
        t.switchScreen("one_player","instance_select");
    };
    document.getElementById("main_twop_button").onclick = function() {
        t.switchScreen("two_player","instance_select");
    }
    document.getElementById("onep_omniscript_back").onclick = function() {
        t.disconnectAll();
        t.drawError("");
        t.switchScreen("main","menu");
    };
    document.getElementById("twop_omniscript_back").onclick = function() {
        t.disconnectAll();
        t.drawError("");
        t.switchScreen("main","menu");
    };
    document.getElementById("onep_omniscript_instance").onchange = function() {
        t.connect(this.value,function(data,con) {
            t.onePlayerConnectionCallback(data,con);
        });
    };
    document.getElementById("twop_omniscript_instance_one").onchange = function() {
        t.connect(this.value,function(data,con) {
            t.connectionOne = con;
            t.twoPlayerCallback(data,con);
        });
    };
    document.getElementById("twop_omniscript_instance_two").onchange = function() {
        t.connect(this.value,function(data,con) {
            t.connectionTwo = con;
            t.twoPlayerCallback(data,con);
        });
    };
    document.getElementById("onep_omniscript_next").onclick = function() {
        if(this.style.color==="white") {
            t.switchScreen("one_player","start_game");
            t.canvas.style.width = (t.canvas.width*2.5)+"px";
            t.canvas.style.height = (t.canvas.height*2.5)+"px";
            var placeholder = document.getElementById("one_player_canvas_placeholder");
            placeholder.appendChild(t.canvas);
            t.canvas.style.display = "block";
        }
    };
    document.getElementById("twop_omniscript_next").onclick = function() {
        if(this.style.color==="white") {
            t.switchScreen("two_player","start_game");
            t.canvas.style.width = (t.canvas.width*2.5)+"px";
            t.canvas.style.height = (t.canvas.height*2.5)+"px";
            var placeholder = document.getElementById("two_player_canvas_placeholder");
            placeholder.appendChild(t.canvas);
            t.canvas.style.display = "block";
        }
    };
    document.getElementById("one_player_start_button").onclick = function() {
        t.start();
        this.style.display = "none";
    };
    document.getElementById("two_player_start_button").onclick = function() {
        t.start();
        this.style.display = "none";
    };
    document.getElementById("one_player_back_button").onclick = function() {
        //Stop any game in progress, sever any connections, return to main menu.
        t.disconnectAll();
        t.drawError("");
        document.getElementById("one_player_start_button").style.display = "block";
        t.switchScreen("main","menu");
    };
    document.getElementById("two_player_back_button").onclick = function() {
        //Stop any game in progress, sever any connections, return to main menu.
        t.disconnectAll();
        t.drawError("");
        document.getElementById("two_player_start_button").style.display = "block";
        t.switchScreen("main","menu");
    };
    window.onbeforeunload = function() {
        //User is leaving the page for one reason or another, lets try to
        //clean up as quick as we can.
        t.disconnectAll();
    };
};
/**
 * This method starts the game.
 */
Pong.prototype.start = function() {
    this.countDown = 5;
    this.p1.position = Math.floor(this.canvas.height/2);
    this.p2.position = Math.floor(this.canvas.height/2);
    this.p1.score = 0;
    this.p2.score = 0;

    this.ball.x = Math.floor(this.canvas.width/2);
    this.ball.y = Math.floor(this.canvas.height/2);

    this.animateCountDown();
};
/**
 * This method starts the countdown after the game has started.
 */
Pong.prototype.animateCountDown = function() {
    var t = this;
    var x;
    if(this.countDown>=0) {
        this.drawBoard();
        this.pushScreen();
        this.countDown--;
        setTimeout(function() {t.animateCountDown();},1000);
    } else {
        this.started = true;
        this.animate();
    }
};
/**
 * This method is responsible for animating the game state
 * per-frame and sending it to the device.
 */
Pong.prototype.animate = function() {
    this.ball.x += this.ball.dx;
    this.ball.y += this.ball.dy;

    if(this.paddleCollision(this.p1)) {
        this.ball.x = this.p1.width+this.ball.radius;
        this.ball.dx *= -1;
    } else if(this.paddleCollision(this.p2)) {
        this.ball.x = this.p2.hPos-this.ball.radius;
        this.ball.dx *= -1;
    }

    if(this.ball.x-this.ball.radius<=0) {
        this.ball.x = this.ball.radius;
        this.ball.dx *= -1;
        this.p2.score++;
    } else if(this.ball.x+this.ball.radius>=this.canvas.width) {
        this.ball.x = this.canvas.width-this.ball.radius;
        this.ball.dx *= -1;
        this.p1.score++;
    }

    if(this.ball.y-this.ball.radius<=0) {
        this.ball.y = this.ball.radius;
        this.ball.dy *= -1;
        
    } else if(this.ball.y+this.ball.radius>=this.canvas.height) {
        this.ball.y = this.canvas.height-this.ball.radius;
        this.ball.dy *= -1;
        
    }

    var x;
    this.drawBoard();
    this.pushScreen();

    if(this.p1.score>=this.scoreLimit || this.p2.score>=this.scoreLimit) {
        this.started = false;
        document.getElementById("one_player_start_button").style.display = "block";
        document.getElementById("two_player_start_button").style.display = "block";
    }
    
    var t = this;
    if(this.started) {
        setTimeout(function() { t.animate(); },this.redraw);
    }
};
/**
 * Given a paddle, this method detects collisions with the
 * game ball.
 */
Pong.prototype.paddleCollision = function(paddle) {
    var bx1 = this.ball.x - this.ball.radius;
    var bx2 = this.ball.x + this.ball.radius;
    var by1 = this.ball.y - this.ball.radius;
    var by2 = this.ball.y + this.ball.radius;
    
    var px1 = paddle.hPos;
    var px2 = paddle.hPos+paddle.width;
    var py1 = paddle.position-Math.floor(paddle.height/2);
    var py2 = paddle.position+Math.floor(paddle.height/2);

    if (bx1 < px2 && bx2 > px1 && by1 < py2 && by2 > py1)  {
        return true;
    } else {
        return false;
    }
};
/**
 * This callback is used by the one player game to receive events from
 * OmniScript.
 */
Pong.prototype.onePlayerConnectionCallback = function(data,con) {
    var t = this;
    var x;
    var connected = document.getElementById("onep_omniscript_connected");
    var instructions = document.getElementById("onep_connect_instructions");
    var pick = document.getElementById("onep_select_device");
    var next = document.getElementById("onep_omniscript_next");
    var list = document.getElementById("onep_select_device_list");
    var instance = document.getElementById("onep_omniscript_instance");

    if(data._class==="WebSocketConnectionEstablished") {
        connected.style.backgroundColor = "green";
        connected.title = "Connected to OmniScript Instance";
        instructions.style.display="none";
        pick.style.display="block";
        next.title="Please select a device to continue.";
        this.drawError("");
    } else if(data._class==="ConnectionOpen") {
        this.redrawDeviceList(list,data.serverInfo.devices,con);
    } else if(data._class==="WebSocketConnectionLost") {
        connected.style.backgroundColor = "red";
        connected.title = "Not connected to an OmniScript Instance";
        instructions.style.display="block";
        pick.style.display="none";
        next.title="Cannot continue until you're connected.";
        instance.value = "";
        while(list.firstChild) { list.removeChild(list.firstChild); }
    } else if(data._class==="DeviceOpenResponse") {
        for(x=0;x<list.childNodes.length;x++) {
            var li = list.childNodes[x];
            if(li.getAttribute("uuid")===data.device.uuid) {
                li.style.fontWeight="bold";
            } else {
                li.style.fontWeight="normal";
            }
        }
        this.connections[this.getConnectionNumber(con)].devConnected = true;
        this.connections[this.getConnectionNumber(con)].connectedUuid = data.device.uuid;
        next.style.color = "white";
        next.title = "";

        this.canvas.width = data.device.displayInfo.width;
        this.canvas.height = data.device.displayInfo.height;
        this.lastFrame.width = this.canvas.width;
        this.lastFrame.height = this.canvas.height;

        this.p1.position = this.canvas.height/2;
        this.p2.position = this.canvas.height/2;

        this.ctx = this.canvas.getContext("2d");

        this.initializeDevice(con,data.device);
        this.drawError("");
    } else if(data._class==="DeviceCloseResponse") {
        for(x=0;x<list.childNodes.length;x++) {
            list.childNodes[x].style.fontWeight="normal";
        }
        this.connections[this.getConnectionNumber(con)].devConnected = false;
        this.connections[this.getConnectionNumber(con)].connectedUuid = "";
        next.style.color = "#606060";
        next.title = "Please select a device to continue.";
    } else if(data._class==="ConnectionDeviceMembershipChange") {
        this.redrawDeviceList(list,data.devices.devices,con);
    } else if(data._class==="PenDown" || data._class==="PenMove" || data._class==="PenUp") {
        var paddle = (data.x<(this.canvas.width/2))?this.p1:this.p2;
        paddle.position = Math.round(data.y);
        if(paddle.position - (paddle.height/2) < 0) {
            paddle.position = Math.floor(paddle.height/2);
        } else if(paddle.position + (paddle.height/2) > this.canvas.height) {
            paddle.position = Math.floor(this.canvas.height - (paddle.height/2));
        }
    } else if(data._class==="ScriptelException") {
        this.drawError(data.message);
        this.started = false;
    } else if(data._class==="RegionUpdateResponse" || data._class==="PushBitmapResponse") {
    } else {
        console.log("Unhandled event: ",data);
    }
};
/**
 * This callback is used by the two player game to handle events
 * coming from OmniScript.
 */
Pong.prototype.twoPlayerCallback = function(data,con) {
    var t = this;
    var x;
    var connectedOne = document.getElementById("twop_omniscript_connected_one");
    var connectedTwo = document.getElementById("twop_omniscript_connected_two");
    var instructions = document.getElementById("twop_connect_instructions");
    var pickOne = document.getElementById("twop_select_device_one");
    var pickTwo = document.getElementById("twop_select_device_two");
    var next = document.getElementById("twop_omniscript_next");
    var listOne = document.getElementById("twop_select_device_list_one");
    var listTwo = document.getElementById("twop_select_device_list_two");
    var instanceOne = document.getElementById("twop_omniscript_instance_one");
    var instanceTwo = document.getElementById("twop_omniscript_instance_two");

    var list = (con===this.connectionOne)?listOne:listTwo;
    var connected = (con===this.connectionOne)?connectedOne:connectedTwo;
    var pick = (con===this.connectionOne)?pickOne:pickTwo;
    var instance = (con===this.connectionOne)?instanceOne:instanceTwo;

    if(data._class==="WebSocketConnectionEstablished") {    
        connected.style.backgroundColor = "green";
        connected.title = "Connected to OmniScript Instance";
        pick.style.display="block";
        if(this.connectionOne && this.connectionTwo) {
            instructions.style.display="none";
            next.title="Please select a device to continue.";
            this.drawError("");
        }
    } else if(data._class==="ConnectionOpen") {
        this.redrawDeviceList(list,data.serverInfo.devices,con);
    } else if(data._class==="WebSocketConnectionLost") {
        connected.style.backgroundColor = "red";
        connected.title = "Not connected to an OmniScript Instance";
        instructions.style.display="block";
        pick.style.display="none";
        next.title="Cannot continue until you're connected.";
        instance.value = "";
        while(list.firstChild) { list.removeChild(list.firstChild); }
    } else if(data._class==="DeviceOpenResponse") {
        for(x=0;x<list.childNodes.length;x++) {
            var li = list.childNodes[x];
            if(li.getAttribute("uuid")===data.device.uuid) {
                li.style.fontWeight="bold";
            } else {
                li.style.fontWeight="normal";
            }
        }
        this.connections[this.getConnectionNumber(con)].devConnected = true;
        this.connections[this.getConnectionNumber(con)].connectedUuid = data.device.uuid;
        next.style.color = "white";
        next.title = "";

        this.canvas.width = data.device.displayInfo.width;
        this.canvas.height = data.device.displayInfo.height;
        this.lastFrame.width = this.canvas.width;
        this.lastFrame.height = this.canvas.height;

        this.p1.position = this.canvas.height/2;
        this.p2.position = this.canvas.height/2;

        this.ctx = this.canvas.getContext("2d");

        this.initializeDevice(con,data.device);
        this.drawError("");
    } else if(data._class==="DeviceCloseResponse") {
        for(x=0;x<list.childNodes.length;x++) {
            list.childNodes[x].style.fontWeight="normal";
        }
        this.connections[this.getConnectionNumber(con)].devConnected = false;
        this.connections[this.getConnectionNumber(con)].connectedUuid = "";
        next.style.color = "#606060";
        next.title = "Please select a device to continue.";
    } else if(data._class==="ConnectionDeviceMembershipChange") {
        this.redrawDeviceList(list,data.devices.devices,con);
    } else if(data._class==="PenDown" || data._class==="PenMove" || data._class==="PenUp") {
        var paddle = (con===this.connectionOne)?this.p1:this.p2;
        paddle.position = Math.round(data.y);
        if(paddle.position - (paddle.height/2) < 0) {
            paddle.position = Math.floor(paddle.height/2);
        } else if(paddle.position + (paddle.height/2) > this.canvas.height) {
            paddle.position = Math.floor(this.canvas.height - (paddle.height/2));
        }
    } else if(data._class==="ScriptelException") {
        this.drawError(data.message);
        this.started = false;
    } else if(data._class==="RegionUpdateResponse" || data._class==="PushBitmapResponse") {
    } else {
        console.log("Unhandled event: ",data);
    }
};
/**
 * This method disconnects from all devices and disconnects all open
 * WebSockets.
 */
Pong.prototype.disconnectAll = function() {
    this.drawDisconnectedScreen();
    this.pushScreen();
    var x;
    this.started = false;
    for(x=0;x<this.connections.length;x++) {
        this.connections[x].connection.close();
    }
};
/**
 * This method draws error text in the error div.
 */
Pong.prototype.drawError = function(msg) {
    var err = document.getElementById("error");
    while(err.firstChild) { err.removeChild(err.firstChild); }
    err.appendChild(document.createTextNode(msg));
};
/**
 * This method redraws a device list during the initial connection screen.
 */
Pong.prototype.redrawDeviceList = function(list,devices,con) {
    var uuid = this.connections[this.getConnectionNumber(con)].connectedUuid;
    var t = this;

    while(list.firstChild) {
        list.removeChild(list.firstChild);
    }
    for(x=0;x<devices.length;x++) {
        var li = document.createElement("li");
        var d = devices[x];
        li.appendChild(document.createTextNode(d.product));
        li.setAttribute("uuid",d.uuid);
        li.onclick = function() {
            if(t.connections[t.getConnectionNumber(con)].devConnected) {
                t.drawDisconnectedScreen();
                t.pushScreen();
                con.send(JSON.stringify({"_class":"DeviceCloseRequest"}));
            }
            //Open the device:
            con.send(JSON.stringify({"_class":"DeviceOpenRequest","uuid":this.getAttribute("uuid")}));
        };
        if(d.uuid === uuid) {
            li.style.fontWeight = "bold";
        }
        list.appendChild(li);
    }
};
/**
 * This method manipulates the HTML div's to switch screens.
 */
Pong.prototype.switchScreen = function(screenId,subScreenId) {
    var screens = document.getElementById("screens");
    var screenElement = document.getElementById(screenId);
    var subScreenElement = document.getElementById(screenId+"_"+subScreenId);

    var x;
    //Start by hiding all screens.
    for(x=0;x<screens.childNodes.length;x++) {
        var cNode = screens.childNodes[x];
        if(cNode.nodeName==="DIV") {
            cNode.style.display = "none";
        }
    }

    //Now hide all subscreens under our screen.
    for(x=0;x<screenElement.childNodes.length;x++) {
        var cNode = screenElement.childNodes[x];
        if(cNode.nodeName==="DIV") {
            cNode.style.display = "none";
        }
    }

    //Now show the screen/subscreen we want.
    screens.style.display="block";
    screenElement.style.display="block";
    subScreenElement.style.display="block";
};
/**
 * This method creates a new WebSocket connection and registers
 * a callback function to handle events for that connection.
 */
Pong.prototype.connect = function(hostStr,callback) {
    if(!hostStr) {
        return;
    }

    var split = hostStr.indexOf(":");
    var host = (split>=0)?hostStr.substring(0,split):hostStr;
    var port = (split>=0)?hostStr.substring(split+1)*1:8080;
    var t = this;
    var url = "ws://"+host+":"+port+"/";
    var con = new WebSocket(url);
    con.onopen = function() {
        //Add this connection to our list.
        t.connections[t.connections.length] = {"connection":con,"devConnected":false,"connectedUuid":""};
        callback({"_class":"WebSocketConnectionEstablished"},con);
    };
    con.onclose = function() {
        //Remove this connection from our list.
        var num = t.getConnectionNumber(con);
        if(num>=0) {
            t.connections.splice(num,1);
        }
        if(con===t.connectionOne) {
            t.connectionOne = false;
        } else if(con===t.connectionTwo) {
            t.connectionTwo = false;
        }
        callback({"_class":"WebSocketConnectionLost"},con);
    };
    con.onerror = function(err) {
        console.log("Connection ",t.getConnectionNumber(con)," (",url,") ran into a problem: ",err);
        callback({"_class":"WebSocketConnectionError","error":err},con);
    };
    con.onmessage = function(data) {
        var msg = JSON.parse(data.data);
        callback(msg,con);
    };
};
/**
 * This method gets the array index of a connection in
 * the main connection array.
 */
Pong.prototype.getConnectionNumber = function(con) {
    var x;
    for(x=0;x<this.connections.length;x++) {
        //Remove this connection.
        if(this.connections[x].connection===con) {
            return x;
        }
    }
    return -1;
};
/**
 * Draws the device disconnected message on the main canvas.
 */
Pong.prototype.drawDisconnectedScreen = function() {
    this.ctx.fillStyle = "white";
    this.ctx.fillRect(0,0,this.canvas.width,this.canvas.height);

    BlockFont.fillText(this.ctx,"Device disconnected from pong, cycle power",Math.round(this.canvas.width/2),Math.round(this.canvas.height/2)-7,"center");
    BlockFont.fillText(this.ctx,"on device to return it to normal.",Math.round(this.canvas.width/2),Math.round(this.canvas.height/2),"center");
};
/**
 * Draws the device ready screen on the main canvas.
 */
Pong.prototype.drawReadyScreen = function() {
    this.ctx.fillStyle = "white";
    this.ctx.textAlign = "center";
    this.ctx.textBaseline = "middle";
    this.ctx.font = "16pt arial";
    this.ctx.fillRect(0,0,this.canvas.width,this.canvas.height);

    this.ctx.fillStyle = "black";
    this.ctx.fillText("Device Connected!",Math.round(this.canvas.width/2),Math.round(this.canvas.height/2));
};
/**
 * This method draws the main game board, used during the game animation.
 */
Pong.prototype.drawBoard = function() {
    //Clear
    this.ctx.fillStyle = "white";
    this.ctx.fillRect(0,0,this.canvas.width,this.canvas.height);

    //Draw Center
    var i;
    this.ctx.strokeStyle = "black";
    this.ctx.beginPath();
    for(i=0;i<(this.canvas.height/6);i++) {
        this.ctx.moveTo((this.canvas.width/2)-0.5,i*6);
        this.ctx.lineTo((this.canvas.width/2)-0.5,((i+1)*6)-1);
    }
    this.ctx.stroke();

    //Scores
    this.ctx.fillStyle = "black";
    this.ctx.lineWidth = "1px";

    //P1
    BlockFont.fillText(this.ctx,this.p1.score+"",Math.round(this.canvas.width/2)-3,4,"right");
    this.ctx.fillRect(this.p1.hPos,this.p1.position-(this.p1.height/2),this.p1.width,this.p1.height);

    //P2
    BlockFont.fillText(this.ctx,this.p2.score+"",Math.round(this.canvas.width/2)+2,4);
    this.ctx.fillRect(this.p2.hPos,this.p2.position-(this.p2.height/2),this.p2.width,this.p2.height);

    //Ball
    this.ctx.beginPath();
    this.ctx.arc(this.ball.x,this.ball.y,this.ball.radius,0,2*Math.PI,false);
    this.ctx.fill();

    if(this.countDown > 0) {
        BlockFont.fillText(this.ctx,"Game is played first to "+this.scoreLimit+".",120,15,"center");
        BlockFont.fillText(this.ctx,"The game is about to begin in "+this.countDown+" seconds.",117,57,"center");
    }
    if(this.p1.score >= this.scoreLimit) {
        BlockFont.fillText(this.ctx,"Player 1 Wins!",120,15,"center");
    } else if(this.p2.score >= this.scoreLimit) {
        BlockFont.fillText(this.ctx,"Player 2 Wins!",120,15,"center");
    }
};
/**
 * This method takes the main canvas, compares it to the last pushed frame,
 * and pushes the differences to the connected ScripTouch devices. Differences
 * are surrounded by a bounding box, cropped down to size and shipped.
 */
Pong.prototype.pushScreen = function() {
    //Get a bounding box around differences:
    var x1 = this.canvas.width;
    var x2 = 0;
    var y1 = this.canvas.height;
    var y2 = 0;

    var nData = this.ctx.getImageData(0,0,this.canvas.width,this.canvas.height).data;
    var oData = this.lastCtx.getImageData(0,0,this.canvas.width,this.canvas.height).data;

    var i,x,y;
    var diff = false;
    for(i=0;i<nData.length;i+=4) {
        if(nData[i] != oData[i]) {
            var pos = i/4;
            x = pos % this.canvas.width;
            y = Math.floor(pos / this.canvas.width);

            if(x < x1) { x1 = x; }
            if(x > x2) { x2 = x; }
            if(y < y1) { y1 = y; }
            if(y > y2) { y2 = y; }

            diff = true;
        }
    }

    if(diff) {
        this.diffFrame.width = (x2-x1)+1;
        this.diffFrame.height = (y2-y1)+1;
        this.diffCtx.drawImage(this.canvas,x1*-1,y1*-1);

        var request = JSON.stringify({
            "_class":"PushBitmapRequest",
            "x":x1,
            "y":y1,
            "data":this.diffFrame.toDataURL("image/png"),
            "bitDepth":1
        });
        for(x=0;x<this.connections.length;x++) {
            if(this.connections[x].devConnected) {
                this.connections[x].connection.send(request);
            }
        }
        this.lastCtx.drawImage(this.canvas,0,0);
    }
};
/**
 * This method gets the device ready for drawing by hiding all
 * visible regions and removing the inkEnabled flag on the main
 * container. This essentially gives us a blank slate to draw on.
 */
Pong.prototype.initializeDevice = function(con,device) {
    this.drawReadyScreen();

    var x;
    //Blank regions.
    con.send(JSON.stringify({
        "_class":"RegionUpdateRequest",
        "region": {
            "id": 2,
            "x1": 0,
            "x2": device.displayInfo.width-1,
            "y1": 0,
            "y2": device.displayInfo.height-1,
            "containerType": {
                "_class":"RegionContainer",
                "flags": {
                    "inkEnabled": false
                }
            },
            "flags": {
                "acceptsTouch": true,
                "decodeRegion": true,
                "hasFrame": false,
                "inUse": true,
                "visible": true
            }
        }
    }));
    for(x=2;x<device.regions.length;x++) {
        con.send(JSON.stringify({
            "_class":"RegionUpdateRequest",
            "region": {
                "id": (x+1),
                "x1": 0,
                "x2": 1,
                "y1": 0,
                "y2": 1,
                "containerType": {"_class":"RegionContainer"},
                "flags": {
                    "acceptsTouch": false,
                    "decodeRegion": false,
                    "hasFrame": false,
                    "inUse": false,
                    "visible": false
                }
            }
        }));
    };

    this.pushScreen();
};