function ScriptelDisplay(canvas, devWidth, devHeight, scale) {
	"use strict";
	this.canvas = canvas;
	this.scale = scale;
	this.devWidth = devWidth;
	this.devHeight = devHeight;

	this.charWidth = 6;
	this.charHeight = 8;

	this.targetSize = 10;

	this.invertButton = -1;

	this.width = this.devWidth * this.scale;
	this.height = this.devHeight * this.scale;

	this.canvas.setAttribute("width", this.width);
	this.canvas.setAttribute("height", this.height);

	this.regionUpdateListeners = [];
	this.selectListeners = [];
	this.activeContainerListeners = [];

	this.selectedRegion = -1;
	this.targetMid = this.targetSize / 2;
	this.targetX1 = 0;
	this.targetY1 = 0;
	this.targetX2 = 0;
	this.targetY2 = 0;
	this.targetMap = false;

	this.dragTarget = 0;
	this.dragLastX = -1;
	this.dragLastY = -1;
	this.dragging = false;

	this.calibrating = false;
	this.calibrationPoint = "";

	this.activeContainer = -1;
	this.lastRegionUpdate = -1;

	this.ctx = this.canvas.getContext("2d");
	this.regions = [];

	this.smoothPenInput = true;
	this.lastPenX = -1;
	this.lastPenY = -1;

	var t = this;
	this.canvas.addEventListener("mousemove", function(evt) { t.onMouseMove(evt); }, false);
	this.canvas.addEventListener("mousedown", function(evt) { t.onMouseDown(evt); }, false);
	this.canvas.addEventListener("mouseup", function(evt) { t.onMouseUp(evt); }, false);
	this.canvas.addEventListener("mouseleave", function(evt) {t.onMouseUp(evt); }, false);
	this.redraw();
}

ScriptelDisplay.prototype.setRegions = function(regions) {
	"use strict";
	this.regions = regions;
	this.lastRegionUpdate = -1;
	this.selectedRegion = -1;
	this.calculateActiveContainer();
	this.redraw();
};

ScriptelDisplay.prototype.setRegion = function(region) {
	"use strict";
	this.regions[region.id - 1] = region;
	this.lastRegionUpdate = region.id - 1;
	this.calculateActiveContainer();
	this.redraw();
};

ScriptelDisplay.prototype.calculateActiveContainer = function() {
	var region;
	var change = false;
	if(this.lastRegionUpdate >= 0) {
		region = this.regions[this.lastRegionUpdate];
		if(region.containerType) {
			change = region.id != this.activeContainer;
			this.activeContainer = region.id;
			if(change) {
				this.notifyActiveContainer();
			}
			return;
		} else if(region.buttonType) {
			change = region.buttonType.parentContainer != this.activeContainer;
			this.activeContainer = region.buttonType.parentContainer;
			if(change) {
				this.notifyActiveContainer();
			}
			return;
		} else if(region.lineType) {
			change = region.lineType.parentContainer != this.activeContainer;
			this.activeContainer = region.lineType.parentContainer;
			if(change) {
				this.notifyActiveContainer();
			}
			return;
		}
	}

	var i;
	for(i = this.regions.length - 1; i >= 0; i--) {
		region = this.regions[i];
		if(region.containerType && region.flags.visible) {
			change = region.id != this.activeContainer;
			this.activeContainer = region.id;
			if(change) {
				this.notifyActiveContainer();
			}
			return;
		}
	}
	this.activeContainer = -1;
};

ScriptelDisplay.prototype.notifyActiveContainer = function() {
	"use strict";
	var i;
	for(i = 0; i < this.activeContainerListeners.length; i++) {
		this.activeContainerListeners[i](this.activeContainer);
	}
};

ScriptelDisplay.prototype.penDown = function(x, y) {
	this.lastPenX = x;
	this.lastPenY = y;
};

ScriptelDisplay.prototype.penUp = function(x, y) {
	this.lastPenX = -1;
	this.lastPenY = -1;
};

ScriptelDisplay.prototype.penMove = function(x, y) {
	if(this.smoothPenInput) {
		this.ctx.lineWidth = 2;
		this.ctx.strokeStyle = "black";
		this.ctx.lineCap = "round";
		this.ctx.beginPath();
		this.ctx.moveTo(this.lastPenX * this.scale, this.lastPenY * this.scale);
		this.ctx.lineTo(x * this.scale, y * this.scale);
		this.ctx.stroke();
	} else {
		this.drawLine(Math.floor(this.lastPenX), Math.floor(this.lastPenY), Math.floor(x), Math.floor(y), "black");
	}

	this.lastPenX = x;
	this.lastPenY = y;
};

ScriptelDisplay.prototype.onMouseMove = function(evt) {
	"use strict";
	if(!this.dragging) {
		var num = this.getTargetNumber(evt);
		if(num) {
			switch(num) {
				case 1:
					this.canvas.style.cursor = "nw-resize";
					break;
				case 2:
					this.canvas.style.cursor = "sw-resize";
					break;
				case 3:
					this.canvas.style.cursor = "ne-resize";
					break;
				case 4:
					this.canvas.style.cursor = "se-resize";
					break;
				case 5:
					this.canvas.style.cursor = "move";
					break;
			}
		} else {
			var region = this.getMouseRegion(evt);
			if(region >= 0) {
				this.canvas.style.cursor = "pointer";
			} else {
				this.canvas.style.cursor = "auto";
			}
		}
	} else {
		var offsetX = Math.floor(evt.offsetX / this.scale) - this.dragLastX;
		var offsetY = Math.floor(evt.offsetY / this.scale) - this.dragLastY;

		if(Math.abs(offsetX) > 0 || Math.abs(offsetY) > 0) {
			switch(this.dragTarget) {
				case 1:
					//Changes X1 and Y1
					this.regions[this.selectedRegion][this.targetMap[1][0]] += offsetX;
					this.regions[this.selectedRegion][this.targetMap[1][1]] += offsetY;
					break;
				case 2:
					//Changes X1 and Y2
					this.regions[this.selectedRegion][this.targetMap[2][0]] += offsetX;
					this.regions[this.selectedRegion][this.targetMap[2][1]] += offsetY;
					break;
				case 3:
					//Changes X2 and Y1
					this.regions[this.selectedRegion][this.targetMap[3][0]] += offsetX;
					this.regions[this.selectedRegion][this.targetMap[3][1]] += offsetY;
					break;
				case 4:
					//Changes X2 and Y2
					this.regions[this.selectedRegion][this.targetMap[4][0]] += offsetX;
					this.regions[this.selectedRegion][this.targetMap[4][1]] += offsetY;
					break;
				case 5:
					//Changes everything
					this.regions[this.selectedRegion].x1 += offsetX;
					this.regions[this.selectedRegion].y1 += offsetY;
					this.regions[this.selectedRegion].x2 += offsetX;
					this.regions[this.selectedRegion].y2 += offsetY;
					break;
			}

			//Quick bounds check to make sure nothing is over
			if(this.regions[this.selectedRegion].x1 < 0) {
				this.regions[this.selectedRegion].x1 = 0;
			} else if(this.regions[this.selectedRegion].x1 >= this.devWidth) {
				this.regions[this.selectedRegion].x1 = this.devWidth - 1;
			}

			if(this.regions[this.selectedRegion].x2 < 0) {
				this.regions[this.selectedRegion].x2 = 0;
			} else if(this.regions[this.selectedRegion].x2 >= this.devWidth) {
				this.regions[this.selectedRegion].x2 = this.devWidth - 1;
			}

			if(this.regions[this.selectedRegion].y1 < 0) {
				this.regions[this.selectedRegion].y1 = 0;
			} else if(this.regions[this.selectedRegion].y1 >= this.devHeight) {
				this.regions[this.selectedRegion].y1 = this.devHeight - 1;
			}

			if(this.regions[this.selectedRegion].y2 < 0) {
				this.regions[this.selectedRegion].y2 = 0;
			} else if(this.regions[this.selectedRegion].y2 >= this.devHeight) {
				this.regions[this.selectedRegion].y2 = this.devHeight - 1;
			}
		}

		this.dragLastX += offsetX;
		this.dragLastY += offsetY;

		var i;
		for(i = 0; i < this.regionUpdateListeners.length; i++) {
			this.regionUpdateListeners[i](this.regions[this.selectedRegion], false);
		}

		this.redraw();
	}
};

ScriptelDisplay.prototype.onMouseDown = function(evt) {
	var num = this.getTargetNumber(evt);
	if(num) {
		this.dragging = true;
		this.dragTarget = num;
		this.dragLastX = Math.floor(evt.offsetX / this.scale);
		this.dragLastY = Math.floor(evt.offsetY / this.scale);

		this.targetMap = this.boundRegion(this.regions[this.selectedRegion]).map;
	} else {
		this.selectedRegion = this.getMouseRegion(evt);
		this.redraw();

		var i;
		for(i = 0; i < this.selectListeners.length; i++) {
			this.selectListeners[i](this.selectedRegion + 1);
		}

		if(this.getTargetNumber(evt)) {
			this.onMouseMove(evt);
			this.onMouseDown(evt);
		}
	}
};

ScriptelDisplay.prototype.onMouseUp = function(evt) {
	if(this.dragging) {
		this.dragging = false;
		this.dragTarget = 0;
		this.onMouseMove(evt);

		var i;
		for(i = 0; i < this.regionUpdateListeners.length; i++) {
			this.regionUpdateListeners[i](this.regions[this.selectedRegion], true);
		}
	}
};

ScriptelDisplay.prototype.redraw = function() {
	this.ctx.fillStyle = "white";
	this.ctx.fillRect(0, 0, this.width, this.height);

	if(!this.calibrating) {
		this.drawRegions();
		this.drawHandles();
	} else {
		this.drawCalibration();
	}
};

ScriptelDisplay.prototype.drawCalibration = function() {
	"use strict";

	this.drawString("CALIBRATING...", 78, 6, "black", "white");
	this.drawString("Touch target with pen.", 50, 50, "black", "white");

	var x = 0, y = 0, width = 9, margin = Math.floor(this.devHeight * 0.10);
	switch(this.calibrationPoint) {
		case "Upper Left":
			x = margin;
			y = margin;
		break;
		case "Lower Left":
			x = margin;
			y = this.devHeight - margin;
		break;
		case "Lower Right":
			x = this.devWidth - margin;
			y = this.devHeight - margin;
		break;
		case "Upper Right":
			x = this.devWidth - margin;
			y = margin;
		break;
		case "Center":
		default:
			x = Math.floor(this.devWidth / 2);
			y = Math.floor(this.devHeight / 2);
		break;
	}

	this.drawLine(x - Math.floor(width / 2), y, x + Math.floor(width / 2), y, "black");
	this.drawLine(x, y - Math.floor(width / 2), x, y + Math.floor(width / 2), "black");
};

ScriptelDisplay.prototype.drawRegions = function() {
	"use strict";

	var i;
	for(i = 0; i < this.regions.length; i++) {
		var r = this.regions[i];
		if(r.flags.visible) {
			if(r.containerType && r.id === this.activeContainer) {
				this.drawContainerRegion(r);
			} else if(r.buttonType && r.buttonType.parentContainer === this.activeContainer) {
				this.drawButtonRegion(r);
			} else if(r.lineType && r.lineType.parentContainer === this.activeContainer) {
				this.drawLineRegion(r);
			}
		}
	}
};

ScriptelDisplay.prototype.getTargetNumber = function(evt) {
	if(evt.offsetX >= (this.targetX1 - this.targetMid) && evt.offsetX <= (this.targetX1 + this.targetMid)) {
		if(evt.offsetY >= (this.targetY1 - this.targetMid) && evt.offsetY <= (this.targetY1 + this.targetMid)) {
			return 1;
		}
		if(evt.offsetY >= (this.targetY2 - this.targetMid) && evt.offsetY <= (this.targetY2 + this.targetMid)) {
			return 2;
		}
	}

	if(evt.offsetX >= (this.targetX2 - this.targetMid) && evt.offsetX <= (this.targetX2 + this.targetMid)) {
		if(evt.offsetY >= (this.targetY1 - this.targetMid) && evt.offsetY <= (this.targetY1 + this.targetMid)) {
			return 3;
		}
		if(evt.offsetY >= (this.targetY2 - this.targetMid) && evt.offsetY <= (this.targetY2 + this.targetMid)) {
			return 4;
		}
	}

	if(evt.offsetX >= this.targetX1 - this.targetMid && evt.offsetX <= this.targetX2 + this.targetMid && evt.offsetY >= this.targetY1 - this.targetMid && evt.offsetY <= this.targetY2 + this.targetMid) {
		return 5;
	}

	return false;
};

ScriptelDisplay.prototype.getMouseRegion = function(evt) {
	var offsetX = Math.floor(evt.offsetX / this.scale);
	var offsetY = Math.floor(evt.offsetY / this.scale);

	var i;
	for(i = this.regions.length - 1; i >= 0; i--) {
		var region = this.regions[i];
		if(region.buttonType && region.buttonType.parentContainer === this.activeContainer) {
			var bounds = this.boundRegion(region);
			if(offsetX >= bounds.x1 && offsetX <= bounds.x2 && offsetY >= bounds.y1 && offsetY <= bounds.y2) {
				return i;
			}
		} else if (region.lineType && region.lineType.parentContainer === this.activeContainer) {
			var bounds = this.boundRegion(region);
			if(offsetX >= bounds.x1 && offsetX <= bounds.x2 && offsetY >= bounds.y1 && offsetY <= bounds.y2) {
				var dist = this.distanceToLine(offsetX, offsetY, region.x1, region.y1, region.x2, region.y2);
				if(dist <= 5) {
					return i;
				}
			}
		}
	}
	return -1;
};

ScriptelDisplay.prototype.distanceToLine = function(pX, pY, lX1, lY1, lX2, lY2) {
	"use strict";

	var sqr = function(x) { return x * x; }
	var dist = function(x1, y1, x2, y2) { return sqr(x2 - x1) + sqr(y2 - y1); }

	var len = dist(lX1, lY1, lX2, lY2);
	if(len === 0) {
		return Math.sqrt(dist(pX, pY, lX1, lY1));
	}

	len = ((pX - lX1) * (lX2 - lX1) + (pY - lY1) * (lY2 - lY1)) / len;

	if (len < 0) {
		return Math.sqrt(dist(pX, pY, lX1, lY1));
	} else if(len > 1) {
		return Math.sqrt(dist(pX, pY, lX2, lY2));
	} else {
		var x = lX1 + len * (lX2 - lX1);
		var y = lY1 + len * (lY2 - lY1);
		return Math.sqrt(dist(pX, pY, x, y));
	}
};

ScriptelDisplay.prototype.drawContainerRegion = function(region) {
	"use strict";

	//Nothing to do really.
};

ScriptelDisplay.prototype.drawLineRegion = function(region) {
	"use strict";
	this.drawLine(region.x1, region.y1, region.x2, region.y2, this.convertColor(region.foregroundColor));
};

ScriptelDisplay.prototype.drawButtonRegion = function(region) {
	"use strict";

	var foreground = this.convertColor(region.foregroundColor);
	var background = this.convertColor(region.backgroundColor);

	if(this.invertButton === region.id) {
		var tmp = foreground;
		foreground = background;
		background = tmp;
	}

	this.fillRect(region.x1, region.y1, region.x2, region.y2, background);
	if(region.flags.hasFrame) {
		this.strokeRect(region.x1, region.y1, region.x2, region.y2, foreground);
	}

	var caption = region.buttonType.caption;
	var strWidth = caption.length * this.charWidth;
	var strHeight = this.charHeight;
	var xM = Math.floor((region.x1 + region.x2) / 2);
	var yM = Math.floor((region.y1 + region.y2) / 2);

	var cX = xM - Math.floor(strWidth / 2) - 1;
	var cY = yM - Math.floor(strHeight / 2);

	this.drawString(region.buttonType.caption, cX, cY, foreground, background);
};

ScriptelDisplay.prototype.drawHandles = function() {
	"use strict";
	
	if(this.selectedRegion >= 0) {
		var region = this.regions[this.selectedRegion];
		var ctx = this.ctx;

		var bounds = this.boundRegion(region);
		this.targetX1 = (bounds.x1 * this.scale) + Math.floor(this.scale / 2);
		this.targetY1 = (bounds.y1 * this.scale) + Math.floor(this.scale / 2);
		this.targetX2 = (bounds.x2 * this.scale) + Math.floor(this.scale / 2);
		this.targetY2 = (bounds.y2 * this.scale) + Math.floor(this.scale / 2);

		var tM = this.targetMid;

		ctx.strokeStyle = "red";
		ctx.fillStyle = "red";

		ctx.beginPath();
		ctx.moveTo(this.targetX1, this.targetY1);
		ctx.lineTo(this.targetX2, this.targetY1);
		ctx.lineTo(this.targetX2, this.targetY2);
		ctx.lineTo(this.targetX1, this.targetY2);
		ctx.lineTo(this.targetX1, this.targetY1);
		ctx.stroke();

		ctx.fillRect(this.targetX1 - tM, this.targetY1 - tM, this.targetSize, this.targetSize);
		ctx.fillRect(this.targetX2 - tM, this.targetY1 - tM, this.targetSize, this.targetSize);
		ctx.fillRect(this.targetX2 - tM, this.targetY2 - tM, this.targetSize, this.targetSize);
		ctx.fillRect(this.targetX1 - tM, this.targetY2 - tM, this.targetSize, this.targetSize);
	} else {
		this.targetX1 = -1;
		this.targetY1 = -1;
		this.targetX2 = -1;
		this.targetY2 = -1;
	}
};

ScriptelDisplay.prototype.boundRegion = function(region) {
	var obj = { "x1": region.x1, "y1": region.y1, "x2": region.x2, "y2": region.y2, "map": {1:["x1","y1"], 2:["x1","y2"], 3:["x2","y1"], 4:["x2","y2"]} };
	var tmp;
	if(obj.x1 > obj.x2) {
		tmp = obj.x1;
		obj.x1 = obj.x2;
		obj.x2 = tmp;

		obj.map[1][0] = "x2";
		obj.map[2][0] = "x2";
		obj.map[3][0] = "x1";
		obj.map[4][0] = "x1";
	}

	if(obj.y1 > obj.y2) {
		tmp = obj.y1;
		obj.y1 = obj.y2;
		obj.y2 = tmp;

		obj.map[1][1] = "y2";
		obj.map[2][1] = "y1";
		obj.map[3][1] = "y2";
		obj.map[4][1] = "y1";
	}
	return obj;
};

ScriptelDisplay.prototype.convertColor = function(colorArr) {
	"use strict";
	return "RGBA(" + colorArr[0] + ", " + colorArr[1] + ", " + colorArr[2] + ", " + 255 + ")";
};

ScriptelDisplay.prototype.fillRect = function(x1, y1, x2, y2, color) {
	"use strict";
	this.ctx.fillStyle = color;
	var cX = (x1 < x2) ? x1 : x2;
	var cY = (y1 < y2) ? y1 : y2;
	var width = Math.abs(x2 - x1) * this.scale;
	var height = Math.abs(y2 - y1) * this.scale;

	this.ctx.fillRect(cX * this.scale, cY * this.scale, width, height);
};

ScriptelDisplay.prototype.strokeRect = function(x1, y1, x2, y2, color) {
	"use strict";
	this.drawLine(x1, y1, x2, y1, color);
	this.drawLine(x1, y2, x2, y2, color);
	this.drawLine(x1, y1, x1, y2, color);
	this.drawLine(x2, y1, x2, y2, color);
};

ScriptelDisplay.prototype.drawLine = function(x1, y1, x2, y2, color) {
	"use strict";
	var deltaX = x2 - x1;
	var deltaY = y2 - y1;

	if(deltaX === 0) {
		while(y1 > y2) {
			this.setPixel(x1, y1--, color);
		}
		while(y1 <= y2) {
			this.setPixel(x1, y1++, color);
		}
	} else if(deltaY === 0) {
		while(x1 > x2) {
			this.setPixel(x1--, y1, color);
		}
		while(x1 <= x2) {
			this.setPixel(x1++, y1, color);
		}
	} else {
		var xySwap = false;
		var tmp;
		if(Math.abs(deltaY) > Math.abs(deltaX)) {
			xySwap = true;
			tmp = x1;
			x1 = y1;
			y1 = tmp;
			tmp = x2;
			x2 = y2;
			y2 = tmp;
		}

		if(x2 - x1 < 0) {
			tmp = x1;
			x1 = x2;
			x2 = tmp;
			tmp = y1;
			y1 = y2;
			y2 = tmp;
		}

		var x, y = y1, e = 0, num = y2 - y1, denom = x2 - x1, threshold = denom / 2;
		for(x = x1; x < x2; x++) {
			if(xySwap) {
				this.setPixel(y, x, color);
			} else {
				this.setPixel(x, y, color);
			}

			e += num;
			if(num < 0) {
				if(e < -threshold) {
					e += denom;
					y--;
				}
			} else if(e > threshold) {
				e -= denom;
				y++;
			}
		}

		if(xySwap) {
			this.setPixel(y, x, color);
		} else {
			this.setPixel(x, y, color);
		}
	}
};

ScriptelDisplay.prototype.setPixel = function(x, y, color) {
	"use strict";
	this.ctx.fillStyle = color;

	var xPos = (x * this.scale);
	var yPos = (y * this.scale);

	this.ctx.fillRect(xPos, yPos, this.scale, this.scale);
};

ScriptelDisplay.prototype.drawChar = function(chr, x, y, foreColor, backColor) {
	"use strict";
	var c = ScriptelDisplay.glyphs[chr.charCodeAt(0)];

	y++;

	var cX = x, cY = y, byteIdx, bitIdx;
	if(c) {
		for(byteIdx = 0; byteIdx < c.length; byteIdx++) {
			for(bitIdx = 7; bitIdx >= 0; bitIdx--) {
				var on = ((c[byteIdx] >> bitIdx) & 1) > 0;
				this.setPixel(cX, cY, on ? foreColor : backColor);

				cX++;
				if(cX > x + this.charWidth + 1) {
					cY++;
					cX = x;
				}
			}
		}
	}
};

ScriptelDisplay.prototype.drawString = function(str, x, y, foreColor, backColor) {
	"use strict";
	var i;
	for(i = str.length - 1; i >= 0; i--) {
		this.drawChar(str.charAt(i), x + (i * this.charWidth), y, foreColor, backColor);
	}
}


ScriptelDisplay.glyphs = [
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x04, 0x04, 0x04, 0x04, 0x00, 0x00, 0x04 ],
	[ 0x0A, 0x0A, 0x0A, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x0A, 0x0A, 0x1F, 0x0A, 0x1F, 0x0A, 0x0A ],
	[ 0x04, 0x0F, 0x10, 0x0E, 0x01, 0x1E, 0x04 ],
	[ 0x18, 0x19, 0x02, 0x04, 0x08, 0x13, 0x03 ],
	[ 0x0C, 0x12, 0x14, 0x08, 0x15, 0x12, 0x0D ],
	[ 0x0C, 0x04, 0x08, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x02, 0x04, 0x08, 0x08, 0x08, 0x04, 0x02 ],
	[ 0x08, 0x04, 0x02, 0x02, 0x02, 0x04, 0x08 ],
	[ 0x00, 0x04, 0x15, 0x0E, 0x15, 0x04, 0x00 ],
	[ 0x00, 0x04, 0x04, 0x1F, 0x04, 0x04, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x0C, 0x04, 0x08 ],
	[ 0x00, 0x00, 0x00, 0x1F, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x0C, 0x0C ],
	[ 0x00, 0x01, 0x02, 0x04, 0x08, 0x10, 0x00 ],
	[ 0x0E, 0x11, 0x13, 0x15, 0x19, 0x11, 0x0E ],
	[ 0x04, 0x0C, 0x04, 0x04, 0x04, 0x04, 0x0E ],
	[ 0x0E, 0x11, 0x01, 0x02, 0x04, 0x08, 0x1F ],
	[ 0x1F, 0x02, 0x04, 0x02, 0x01, 0x11, 0x0E ],
	[ 0x02, 0x06, 0x0A, 0x12, 0x1F, 0x02, 0x02 ],
	[ 0x1F, 0x10, 0x1E, 0x01, 0x01, 0x11, 0x0E ],
	[ 0x06, 0x08, 0x10, 0x1E, 0x11, 0x11, 0x0E ],
	[ 0x1F, 0x01, 0x02, 0x04, 0x04, 0x04, 0x04 ],
	[ 0x0E, 0x11, 0x11, 0x0E, 0x11, 0x11, 0x0E ],
	[ 0x0E, 0x11, 0x11, 0x0F, 0x01, 0x02, 0x0C ],
	[ 0x00, 0x0C, 0x0C, 0x00, 0x0C, 0x0C, 0x00 ],
	[ 0x00, 0x0C, 0x0C, 0x00, 0x0C, 0x04, 0x08 ],
	[ 0x02, 0x04, 0x08, 0x10, 0x08, 0x04, 0x02 ],
	[ 0x00, 0x00, 0x1F, 0x00, 0x1F, 0x00, 0x00 ],
	[ 0x08, 0x04, 0x02, 0x01, 0x02, 0x04, 0x08 ],
	[ 0x0E, 0x11, 0x01, 0x02, 0x04, 0x00, 0x04 ],
	[ 0x0E, 0x11, 0x01, 0x0D, 0x15, 0x15, 0x0E ],
	[ 0x0E, 0x11, 0x11, 0x11, 0x1F, 0x11, 0x11 ],
	[ 0x1E, 0x09, 0x09, 0x0E, 0x09, 0x09, 0x1E ],
	[ 0x0E, 0x11, 0x10, 0x10, 0x10, 0x11, 0x0E ],
	[ 0x1E, 0x09, 0x09, 0x09, 0x09, 0x09, 0x1E ],
	[ 0x1F, 0x10, 0x10, 0x1E, 0x10, 0x10, 0x1F ],
	[ 0x1F, 0x10, 0x10, 0x1E, 0x10, 0x10, 0x10 ],
	[ 0x0E, 0x11, 0x10, 0x13, 0x11, 0x11, 0x0F ],
	[ 0x11, 0x11, 0x11, 0x1F, 0x11, 0x11, 0x11 ],
	[ 0x0E, 0x04, 0x04, 0x04, 0x04, 0x04, 0x0E ],
	[ 0x07, 0x02, 0x02, 0x02, 0x02, 0x12, 0xC0 ],
	[ 0x11, 0x12, 0x14, 0x18, 0x14, 0x12, 0x11 ],
	[ 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x1F ],
	[ 0x11, 0x1B, 0x15, 0x15, 0x11, 0x11, 0x11 ],
	[ 0x11, 0x19, 0x19, 0x15, 0x13, 0x13, 0x11 ],
	[ 0x0E, 0x11, 0x11, 0x11, 0x11, 0x11, 0x0E ],
	[ 0x1E, 0x11, 0x11, 0x1E, 0x10, 0x10, 0x10 ],
	[ 0x0E, 0x11, 0x11, 0x11, 0x15, 0x12, 0x0D ],
	[ 0x1E, 0x11, 0x11, 0x1E, 0x14, 0x12, 0x11 ],
	[ 0x0E, 0x11, 0x10, 0x0E, 0x01, 0x11, 0x0E ],
	[ 0x1F, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04 ],
	[ 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x0E ],
	[ 0x11, 0x11, 0x11, 0x11, 0x11, 0x0A, 0x04 ],
	[ 0x11, 0x11, 0x11, 0x15, 0x15, 0x1B, 0x11 ],
	[ 0x11, 0x11, 0x0A, 0x04, 0x0A, 0x11, 0x11 ],
	[ 0x11, 0x11, 0x11, 0x0A, 0x04, 0x04, 0x04 ],
	[ 0x1F, 0x01, 0x02, 0x04, 0x08, 0x10, 0x1F ],
	[ 0x0E, 0x08, 0x08, 0x08, 0x08, 0x08, 0x0E ],
	[ 0x00, 0x10, 0x08, 0x04, 0x02, 0x01, 0x00 ],
	[ 0x0E, 0x02, 0x02, 0x02, 0x02, 0x02, 0x0E ],
	[ 0x04, 0x0A, 0x11, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1F ],
	[ 0x10, 0x08, 0x04, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x0E, 0x01, 0x0F, 0x11, 0x0F ],
	[ 0x10, 0x10, 0x16, 0x19, 0x11, 0x11, 0x1E ],
	[ 0x00, 0x00, 0x0E, 0x11, 0x10, 0x11, 0x0E ],
	[ 0x01, 0x01, 0x0D, 0x13, 0x11, 0x11, 0x0F ],
	[ 0x00, 0x00, 0x0E, 0x11, 0x1F, 0x10, 0x0E ],
	[ 0x02, 0x05, 0x04, 0x0E, 0x04, 0x04, 0x04 ],
	[ 0x00, 0x0D, 0x13, 0x13, 0x0D, 0x01, 0x0E ],
	[ 0x10, 0x10, 0x16, 0x19, 0x11, 0x11, 0x11 ],
	[ 0x04, 0x00, 0x0C, 0x04, 0x04, 0x04, 0x0E ],
	[ 0x02, 0x00, 0x06, 0x02, 0x02, 0x12, 0x0C ],
	[ 0x08, 0x08, 0x09, 0x0A, 0x0C, 0x0A, 0x09 ],
	[ 0x0C, 0x04, 0x04, 0x04, 0x04, 0x04, 0x0E ],
	[ 0x00, 0x00, 0x1A, 0x15, 0x15, 0x15, 0x15 ],
	[ 0x00, 0x00, 0x16, 0x19, 0x11, 0x11, 0x11 ],
	[ 0x00, 0x00, 0x0E, 0x11, 0x11, 0x11, 0x0E ],
	[ 0x00, 0x16, 0x19, 0x19, 0x16, 0x10, 0x10 ],
	[ 0x00, 0x0D, 0x13, 0x13, 0x0D, 0x01, 0x01 ],
	[ 0x00, 0x16, 0x19, 0x10, 0x10, 0x10, 0x10 ],
	[ 0x00, 0x00, 0x0F, 0x10, 0x1E, 0x01, 0x1F ],
	[ 0x08, 0x08, 0x1C, 0x08, 0x08, 0x09, 0x06 ],
	[ 0x00, 0x00, 0x12, 0x12, 0x12, 0x12, 0x0D ],
	[ 0x00, 0x00, 0x11, 0x11, 0x11, 0x0A, 0x04 ],
	[ 0x00, 0x00, 0x11, 0x11, 0x15, 0x15, 0x0A ],
	[ 0x00, 0x00, 0x11, 0x0A, 0x04, 0x0A, 0x11 ],
	[ 0x00, 0x11, 0x11, 0x13, 0x0D, 0x01, 0x0E ],
	[ 0x00, 0x00, 0x1F, 0x02, 0x04, 0x08, 0x1F ],
	[ 0x02, 0x04, 0x04, 0x08, 0x04, 0x04, 0x02 ],
	[ 0x04, 0x04, 0x04, 0x00, 0x04, 0x04, 0x04 ],
	[ 0x08, 0x04, 0x04, 0x02, 0x04, 0x04, 0x08 ],
	[ 0x08, 0x15, 0x02, 0x00, 0x00, 0x00, 0x00 ],
	[ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 ]
];

function Region() {
    "use strict";
    this.id = -1;
    this.buttonType = undefined;
    this.containerType = undefined;
    this.lineType = undefined;
    this.flags = undefined;
    this.x1 = -1;
    this.y1 = -1;
    this.x2 = -1;
    this.y2 = -1;
    this.foregroundColor = [];
    this.backgroundColor = [];
    this._class = "Region";
}

function RegionFlags() {
    "use strict";
    this.acceptsTouch = false;
    this.decodeRegion = false;
    this.visible = false;
    this.inUse = false;
    this.hasFrame = false;
    this._class = "RegionFlags";
}

function RegionContainer() {
    "use strict";
    this.flags = undefined;
    this._class = "RegionContainer";
}

function RegionContainerFlags() {
    "use strict";
    this.inkEnabled = false;
}

function RegionButton() {
    "use strict";
    this.parentContainer = -1;
    this.flags = undefined;
    this.caption = "";
    this._class = "RegionButton";
}

function RegionButtonFlags() {
    "use strict";
    this.enabled = false;
    this.highlightOnSelect = false;
    this.clearOnSelect = false;
    this.confirmOnSelect = false;
}

function RegionLine() {
    "use strict";
    this.parentContainer = -1;
    this.lineWidth = -1;
    this._class = "RegionLine";
}